 /*
 * @las3Read.java Version 1.1 03/05/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las3.io;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;                   // Global Common Class Data Structure

import iqstrat.iqstratStruct;           // Global IQSTRAT Data Structure
import iqstrat.iqstratHeadersStruct;    // Well Headers Information Structure
import iqstrat.iqstratControlStruct;    // Control Data Structure
import iqstrat.iqstratRemarkListStruct; // Remarks Data List Structure

import las.lasFileDataStruct;           // LAS File Data Structure

import las3.las3ListStruct;             // LAS Version 3 File Data List Structure
import las3.las3Struct;                 // LAS Version 3 File Data Structure
import las3.las3Curves;                 // Allow User to Change Selected Curves
import las3.io.las3ReadFile;            // Open, Read & Parse LAS Files

import horizon.strat.stratListStruct;   // Stratigraphic Units List Structure

import rock.rockDataListStruct;         // Core Data List Structure

import util.utilWaitAnimation;          // Utility Wait Animation Class
import util.SwingWorker;                // Utility Swing Worker Class

/** Class las3Read
 *  <p> This Class is designed to open a LAS File either Version 2 or 3 and
 *      parse the file into Profile Data Structures
 *
 *  @version 1.1 03/05/2013
 *  @author  John R. Victorine
 */

public class las3Read
{
  // Input Variables

  private cmnStruct     stCMN     = null; // Global Common Data Structure
  private iqstratStruct stStruct  = null; // Global IQSTRAT Data Structure

  // Global Variables

  public static final int _LOAD_DATA = 0;
  public static final int _LOAD_LAS3 = 1;

  private int iLoad = _LOAD_DATA;

  private String sReadMethod  = "";  // File Read Method
  private String sDirectory   = "";  // Directory Path to the LAS File
  private String sFilename    = "";  // File name of LAS File

  private las3ReadFile            p3         = null; // Read LAS File
  private las3Curves              pCurves    = null; // Choose Log Curves

  // -- Well/Outcrop Data

  private iqstratHeadersStruct    stHeader   = null; // Headers Data Structure
  private iqstratControlStruct    stControl   = null; // Control Data Structure

  private las3ListStruct          stLAS3Data = null; // LAS 3 Data List Struct
  private lasFileDataStruct       stLASData  = null; // LAS File Data Struct

  private stratListStruct         stTopsData = null; // Tops Data List Struct

  private rockDataListStruct      stRockData = null; // Rock Data List Struct

  private iqstratRemarkListStruct stRemarks  = null; // Remarks Data List Struct

  private double                  dStart     = 0.0;  // Starting LAS File Depth
  private double                  dEnd       = 0.0;  // Ending LAS File Depth

  /** Construct las3Read()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   */

  public las3Read( iqstratStruct stStruct )
  {
    try
    {
      this.stStruct  = stStruct;
      this.stCMN     = stStruct.stCMN;
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** Construct las3Read()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   *  @param iLoad     = User defined Load or not Load Profile data
   */

  public las3Read( iqstratStruct stStruct, int iLoad )
  {
    try
    {
      this.stStruct  = stStruct;
      this.stCMN     = stStruct.stCMN;
      this.iLoad     = iLoad;
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /* ===================================================================== *
   * ------------------------- DELETE METHODS ---------------------------- *
   * ===================================================================== */

  /** Method delete()
   * <p> This method will set all the structures to null to force java to free
   *     the allocated memory
   */

  public void delete()
  {
    // Input Variables

    stCMN     = null;  // GLobal Common Data Structure
    stStruct  = null;  // Global IQSTRAT Data Structure

    // Global Variables

    sReadMethod  = null;  // File Read Method
    sDirectory   = null;  // Directory Path to the LAS File
    sFilename    = null;  // File name of LAS File

    // Delete Data

    clearData();

    // Delete LAS 3 Read File

    if (p3 != null)
      p3.delete();
    p3 = null;

    // Delete Selected Log Curves Frame

    if (pCurves != null)
      pCurves.close();
    pCurves = null;
  }

  /** Method clearData()
   * <p> This method will set all the data structures to null to force
   *     java to free the allocated memory
   */

  public void clearData()
  {
    dStart = 0.0; // Starting LAS File Depth
    dEnd   = 0.0; // Ending LAS File Depth

    // Remove the LAS 3 File Data Structure

    if (stLAS3Data != null)
      stLAS3Data.delete();
    stLAS3Data = null;

    // Remove the Well Header Information Data Structure

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;

    // Remove the LAS File Data Structure

    if (stLASData != null)
    {
      stLASData.delete();
      stLASData = null;
    }

    if (stTopsData != null)
      stTopsData.delete();
    stTopsData = null;

    if (stRockData != null)
      stRockData.delete();
    stRockData = null;

    if (stControl != null)
      stControl.delete();
    stControl = null;

    if (stRemarks != null)
      stRemarks.delete();
    stRemarks = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getPath()
   * <p> This method will search for a LAS File that is on the user's PC and
   *     load the LAS File into the data structures.
   */

  public void getPath()
  {
    FileDialog dialog    = null;
    Frame      frameTemp = new Frame();

    dialog = new FileDialog( frameTemp, "Select LAS File from your PC." );
    dialog.setVisible( true );
    frameTemp.setVisible( true );

    // Retrieve LAS File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      setPath( "FILE", dialog.getDirectory(), dialog.getFile() );
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /** Method getData()
   *  <p> This method will load the LAS 3.0 Data into the Data Structures
   */

  public void getData()
  {
    stLASData = las.lasFileDataUtility.transfer( pCurves.getLAS() );
    stLASData = las3.las3LoadLogData.modifyLogData( stLASData );
    stLASData = las.lasFileDataUtility.computeMath( stLASData );

    stControl  = iqstrat.iqstratControlUtility.transfer( pCurves.getControl() );

    stTopsData = horizon.strat.stratUtility.transfer( pCurves.getTops() );

    stRockData = rock.rockDataUtility.transfer( pCurves.getRockData() );

    stRemarks  = iqstrat.iqstratRemarkUtility.transfer( pCurves.getRemarks() );

    if (pCurves != null)
      pCurves.close();
    pCurves = null;
  }

  /** Method getStartingDepth()
   * <p> This method will retrieve the starting depth of the LAS File.
   * @return dStart = the starting depth of the LAS File.
   */

  public double getStartingDepth() { return (dStart); }

  /** Method getEndingDepth()
   * <p> This method will retrieve the ending depth of the LAS File.
   * @return dStart = the ending depth of the LAS File.
   */

  public double getEndingDepth() { return (dEnd); }

  /** Method getLAS3Data()
   * <p> This method will retrieve the LAS 3 Raw Data List Structure
   *     This data structure contains all the data sections that are present
   *     in the LAS 3 File or just the Log Data Section for the LAS 2 File.
   * @return stLAS3Data = the LAS 3 Raw Data List Structure
   */

  public las3ListStruct getLAS3Data()
  {
    return (las3.las3Utility.removeIQSTRAT( stLAS3Data ));
  }

  /** Method getWellData()
   * <p> This method will retrieve the Well Information Data Structure
   * @return stHeader = the Well Information Data Structure
   */

  public iqstratHeadersStruct getWellData() { return (stHeader); }

  /** Method getControlData()
   * <p> This method will retrieve the Plot Control Data Structure
   * @return stControl = the Plot Control Data Structure
   */

  public iqstratControlStruct getControlData() { return (stControl); }

  /** Method getLASData()
   * <p> This method will retrieve the LAS File Data Structure
   * @return stLASData = the LAS File Data Structure
   */

  public lasFileDataStruct getLASData() { return (stLASData); }

  /** Method getTops()
   * <p> This method will retrieve the Stratigraphic Unit Data List Structure
   * @return stTopsData = the Stratigraphic Unit Data List Structure
   */

  public stratListStruct getTops() { return (stTopsData); }

  /** Method getRockData()
   * <p> This method will retrieve the Core Data List Structure
   * @return stRockData = the Core Data List Structure
   */

  public rockDataListStruct getRockData() { return (stRockData); }


  /** Method getRemarks()
   * <p> This method will retrieve the Remarks Data List Structure
   * @return stRemarks = the Remarks Data List Structure
   */

  public iqstratRemarkListStruct getRemarks() { return (stRemarks); }

  /* =========================== ========================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setPath()
   * <p> This method will search for a LAS File that is on the user's PC and
   *     load the LAS File into the data structures.
   * @param sMethod    = the type of file read method, i.e. "FILE" or "URL"
   * @param sDirectory = the Directory or URL Path to LAS File
   * @param sFilename  = the Filename of the LAS File
   */

  public void setPath( String sMethod, String sDirectory, String sFilename )
  {
    // Retrieve LAS File Location

    if ((sMethod != null) && (sDirectory != null) && (sFilename != null))
    {
      this.sReadMethod = new String( sMethod );
      this.sDirectory  = new String( sDirectory );
      this.sFilename   = new String( sFilename );

      ReadLAS3(); //WaitReadLASFile();
    }
  }

  /* ===================================================================== *
   * -------------------------- READ METHODS ----------------------------- *
   * ===================================================================== */

  /** Method WaitReadLASFile()
   * <p> This method will display the Wait Animation for opening and parsing a
   *     digtial LAS File.
   */

  private void WaitReadLASFile()
  {
    java.net.URL url = null;

    // Get the URL Path to display the Wait Animation

    try
    {
      url = new java.net.URL(cmn.cmnStruct.IQSTRAT_BASE + "/");
    }
    catch(Exception ex)
    {
      System.out.println("Wrong Image Directory" + ex);
    }

    // Start the Wait Animation Dialog

    final utilWaitAnimation pDialog = new utilWaitAnimation(url);
    pDialog.startAnimation();

    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        ReadLAS3();

        return "OK";
      }

      //Runs on the event-dispatching thread.

      public void finished() { pDialog.stopAnimation(); }
    };

    // Start the Wait Animation Thread

    worker.start();  //required for SwingWorker
  }

  /** Method ReadLAS3()
   * <p> This method will load and print the LAS 3 File Data
   */

  private void ReadLAS3()
  {
    int iType = util.utilFileIO.DIRECTORY;

    if (this.sReadMethod.equals("URL"))
      iType = util.utilFileIO.URL_FILE;

    // Clear all Data Structures - for new LAS file Read

    clearData();

    // Open LAS 3 File and Read Data

    p3 = new las3ReadFile();

    p3.Read( iType, this.sDirectory, this.sFilename );
    stLAS3Data = las3.las3Utility.copyList( p3.getData() );

    if (iLoad == _LOAD_DATA)
      loadLAS3( stLAS3Data );

    // Deallocate Java Memory

    if (p3 != null) // Close & clear memory for the LAS 3 File Read Class
      p3.delete();
    p3 = null;
  }

  /* ===================================================================== *
   * -------------------------- LOAD METHODS ----------------------------- *
   * ===================================================================== */

  /** Method loadLAS3()
   * <p> This method will load the LAS 3 Data List Structure into the Profile
   *     Data List Structures
   * @param st = the LAS 3 File Data List Structure
   */

  private void loadLAS3( las3ListStruct st )
  {
    double dTemp = 0.0;

    // Load Well Headers Data Structure

    stHeader = las3.las3LoadWellHeader.getData( st );

    // Load Required LAS File Data Structure Fields

    stLASData = las3.las3LoadLogData.getData( st );

    // Add Version Number, set default plot depth range

    if (stLASData != null)
    {
      stLASData.sReadMethod = new String( sReadMethod ); // Method
      stLASData.sDirectory  = new String( sDirectory );  // Directory
      stLASData.sFilename   = new String( sFilename );   // Filename
      stLASData.sVersion    = new String( "" + st.dVersion );  // Version No
      dStart                = stLASData.depthStart; // Starting Depth
      dEnd                  = stLASData.depthEnd;   // Ending Depth

      if (dEnd < dStart)
      {
        dTemp  = dEnd;
        dEnd   = dStart;
        dStart = dTemp;
      }
    }

    // Set the missing depth variables depending on the source of data

    stHeader = las3.las3LoadWellHeader.normalizeDepth( stHeader, stLASData );
    stLASData = las3.las3LoadLogData.normalizeDepth( stHeader, stLASData );

    // Open Curve Selection Dialog to give the user a chance to change the
    // default curve selections.  This Dialog displays all PROFILE Web App
    // data curves that are read from a LAS File Version 2.0 or 3.0.

    pCurves = new las3Curves( stStruct, stLASData, st );
  }
}

/*
 *  @version 1.1 03/05/2013
 *  @author  John Victorine
 */
