/*
 * @las3Curves.java Version 1.1 03/05/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las3;

import java.io.*;
import java.awt.*;
//import java.awt.event.*;

//import javax.swing.*;
//import javax.swing.event.*;

import iqstrat.iqstratStruct;           // Global IQSTRAT Data Structure
import iqstrat.iqstratControlStruct;    // Plot Control Data Structure
import iqstrat.iqstratRemarkListStruct; // Remarks/Comments/Notes List Structure

import horizon.strat.stratListStruct;   // Stratigraphic Units List Structure
import horizon.strat.stratStruct;       // Stratigraphic Units Data Structure

import rock.rockDataListStruct;         // Rock Data List Structure
import rock.rockDataStruct;             // Rock Data Structure

import las3.las3ListStruct;             // LAS 3 Data List Structure
import las3.las3Struct;                 // LAS 3 Data Structure
import las.lasFileDataStruct;           // LAS File Data Structure
import las3.las3CurvesLog;              // LAS 3 Curves Log Panel

/** Class las3Curves
 *  <p> This Class will display the Curves sections of the LAS 3 File and allow
 *      the user to select the curves they wish to display on the KGS web apps
 *      and to identify unknown curves with KGS Standard Curves.
 *
 *  @version 1.1 03/05/2013
 *  @author  John R. Victorine
 */

public class las3Curves
{
  // Input Variables

  private iqstratStruct      stStruct  = null; // Global IQSTRAT Data Structure
  private las3ListStruct     stLAS3    = null; // LAS 3 File Data List Structure
  private lasFileDataStruct  stLAS     = null; // LAS File Data Structure

  // Global Variables

  private int                     iqstrato    = -1;

  private String                  sReadMethod = "";  // File Read Method
  private String                  sDirectory  = "";  // Directory Path
  private String                  sFilename   = "";  // File name of LAS File

  private int                     iLogs       = 0;
  private lasFileDataStruct       stLogs[]    = null; // LAS File Data Structure

  private int                     iRock       = 0;
  private rockDataListStruct      stRock[]    = null; // Rock Data List Structure

  private int                     iIQTops     = -1;
  private int                     iTops       = 0;
  private stratListStruct         stTops[]    = null; // Stratigraphic Units List

  private int                     iControl    = 0;
  private iqstratControlStruct    stControl[] = null; // Control Data Structure

  private int                     iRemarks    = 0;
  private iqstratRemarkListStruct stRemarks[] = null; // Remarks List Structure

  private int                     iTotal      = 0;    // Total Number of Panels
  private las3CurvesLog           pCurves[]   = null; // Curve Selection Panel
  private String                  sCurves[]   = null;
  private int                     iData[][]   = null; // LAS 3 Order, Type, Array

  // Use either step depth
  public static final int _SAME   = 0; // Both LAS Files have same step value
  // One Step Depth is greater than the other use the greater value as step depth
  public static final int _STEP_1 = 1; // Step value for 1 is less than 2
  public static final int _STEP_2 = 2; // Step value for 2 is less than 1
  // Both have 1 ft depth step, but one starts at 1/2 foot and the other
  // starts at a 1 foot depth value select the second file for interpolation.
  public static final int _SAME_I = 3; // Both LAS Files have same step value
  // Both have .5 or less step size, but one starts at a different depth than
  // the other need to normalize the depth
  public static final int _SAME_S = 4; // Both LAS Files have same step value

  private int    iStep  = _SAME;

  public static final int _NONE          = las3.las3Constants._NONE;

  public static final int _VERSION       = las3.las3Constants._VERSION;
  public static final int _WELL          = las3.las3Constants._WELL;
  public static final int _LOG           = las3.las3Constants._LOG;
  public static final int _CORE          = las3.las3Constants._CORE;
  public static final int _TOPS          = las3.las3Constants._TOPS;
  public static final int _IQ_CONTROL    = las3.las3Constants._IQ_CONTROL;
  public static final int _IQ_GEO_REPORT = las3.las3Constants._IQ_GEO_REPORT;

  public static final int _LAS_TYPE      = las.gui.lasToolsTable._LAS_TYPE;
  public static final int _CORE_TYPE     = las.gui.lasToolsTable._CORE_TYPE;
  public static final int _TOPS_TYPE     = las.gui.lasToolsTable._TOPS_TYPE;
  public static final int _CNTRL_TYPE    = las.gui.lasToolsTable._CNTRL_TYPE;
  public static final int _GEO_TYPE      = las.gui.lasToolsTable._GEO_TYPE;

  /** Constructor las3Curves()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = Global IQSTRAT Data Structure
   *  @param stLAS     = LAS File Data Structure
   *  @param stLAS3    = LAS 3 File Data List Structure
   */

  public las3Curves( iqstratStruct     stStruct,
                     lasFileDataStruct stLAS,
                     las3ListStruct    stLAS3 )
  {
    try
    {
      this.stStruct  = stStruct;
      this.stLAS     = stLAS;
      this.stLAS3    = stLAS3;

      if (stLAS != null)
      {
        sReadMethod = new String( stLAS.sReadMethod );  // File Read Method
        sDirectory  = new String( stLAS.sDirectory );   // Directory Path
        sFilename   = new String( stLAS.sFilename );    // File name of LAS File
      }

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    int i=0;
    int j=0;
    int iCount = 0;

    // Add Log Panel

    if (stLAS3 != null)
    {
      for (i=0; i<stLAS3.iCount; i++)
      {
        switch (stLAS3.stItem[i].iType)
        {
          case _NONE:
          case _VERSION:       // Version Informaiton
          case _WELL:          // Well Information
            break;
          case _LOG:           // Log Data Section
            iLogs++;
            iTotal++;
            break;
          case _CORE:          // Core Data Section
            iRock++;
            iTotal++;
            break;
          case _TOPS:          // Tops Data Section
            if (stLAS3.stItem[i].iqstrat == 1) iIQTops = 1;
            iTops++;
            iTotal++;
            break;
          case _IQ_CONTROL:
            iControl++;
            iTotal++;
            break;
          case _IQ_GEO_REPORT:
            iRemarks++;
            iTotal++;
            break;
        }
      }

      if (iLogs > 0)
      {
        stLogs = new lasFileDataStruct[iLogs];
        for (i=0; i<iLogs; i++)
          stLogs[i] = new lasFileDataStruct();
      }

      if (iRock > 0)
      {
        stRock = new rockDataListStruct[iRock];
        for (i=0; i<iRock; i++)
          stRock[i] = new rockDataListStruct();
      }

      if (iTops > 0)
      {
        stTops = new stratListStruct[iTops];
        for (i=0; i<iTops; i++)
          stTops[i] = new stratListStruct();
      }

      if (iControl > 0)
      {
        stControl = new iqstratControlStruct[iControl];
        for (i=0; i<iControl; i++)
          stControl[i] = new iqstratControlStruct();
      }

      if (iRemarks > 0)
      {
        stRemarks = new iqstratRemarkListStruct[iRemarks];
        for (i=0; i<iRemarks; i++)
          stRemarks[i] = new iqstratRemarkListStruct();
      }

      if (iTotal > 0)
      {
        pCurves = new las3CurvesLog[iTotal];
        sCurves = new String[iTotal];
        iData   = new int[iTotal][3];   // LAS 3 Order, Type, Array

        for (i=0; i<iTotal; i++)
        {
          sCurves[i] = new String("");
          for (j=0; j<3; j++)
            iData[i][j] = -1;
        }

        for (i=0; i<stLAS3.iCount; i++)
        {
          iqstrato = -1;
          switch (stLAS3.stItem[i].iType)
          {
            case _LOG:         // Log Data Section
            case _CORE:        // Core Data Section
              break;
            case _TOPS:        // Tops Data Section
              if (iIQTops == 1) iqstrato = 1;
              break;
          }

          switch (stLAS3.stItem[i].iType)
          {
            case _LOG:           // Log Data Section
            case _CORE:        // Core Data Section
            case _TOPS:          // Tops Data Section
            case _IQ_CONTROL:    // PROFILE - Control Plot Data
            case _IQ_GEO_REPORT: // PROFILE - Remarks/Comments/Notes
              if (iCount < iTotal)
              {
                pCurves[iCount] = new las3CurvesLog(
                    stStruct, stLAS, stLAS3.dVersion, iqstrato, stLAS3.stItem[i] );

                if (pCurves[iCount] != null)
                {
                  sCurves[iCount] = new String( pCurves[iCount].getLabel() );
                  iData[iCount][0] = i;
                  iData[iCount][1] = pCurves[iCount].getTypeID();
                  iData[iCount][2] = pCurves[iCount].getArray();
                }

                iCount++;
              }
              break;
          }
        }
      }
    }
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getLAS3()
   * <p> This method will retrieve the LAS & Core Curve Selection and modify
   *     the LAS 3 File Data List Sturcture to retain the users selection and
   *     redefinition of the curve mnemonics to the KGS mnemonics
   * @return stList = modified LAS 3 File Data List Sturcture
   *

  public las3ListStruct getLAS3()
  {
    las3ListStruct stList      = null;
    las3Struct     st          = null;
    int            iSelected[] = null;

    stList = las3.las3Utility.copyList( stLAS3 );

    for (int i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _LAS_TYPE)
        {
          st     = las3.las3Utility.copy( pCurves[i].getLAS3() );
          stList = las3.las3Utility.modify( st, stList );
        }

        st.delete();
      }
    }

    return (stList);
  }
*/
  /** Method getLAS()
   * <p> This method will retrieve the LAS File Data Structure
   * @return stLAS = the LAS File Data Structure
   */

  public lasFileDataStruct getLAS()
  {
    int               i      = 0;
    int               iCount = 0;
    int               iRows  = 0;
    lasFileDataStruct st     = null;
    double            dNull  = 0.0;
    int               iTemp  = 0;
    double            dStep  = 0.0;
    double            dLogMin = 0.0;
    double            dLogMax = 0.0;

    iTemp            = (int) stLAS.depthStart;
    stLAS.depthStart = (double) iTemp;

    iTemp            = 1 + (int) stLAS.depthEnd;
    stLAS.depthEnd   = (double) iTemp;

    dStep = 0.5;
    if (stLAS.depthStep < 0.5)
      dStep = 0.25;
    if (stLAS.depthStep < 0.25)
      dStep = 0.1;

    stLAS.depthStep  = dStep;

    iRows = 1+(int)(Math.abs(stLAS.depthEnd-stLAS.depthStart)/stLAS.depthStep);

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (iCount < iLogs)
        {
          if (pCurves[i].getDataType() == _LAS_TYPE)
          {
            if (stLAS.iLogs == 0)
            {
              stLAS.sReadMethod = new String( sReadMethod ); // File Read Method
              stLAS.sDirectory  = new String( sDirectory );  // Directory Path
              stLAS.sFilename   = new String( sFilename );   // File name of LAS File

              stLAS.stLAS3      = new las3Struct[iLogs];
              stLAS.iLogs       = iLogs;

              stLAS = las3.las3LoadLogData.initData( iRows, stLAS );
            }

            stLAS = pCurves[i].getLAS( stLAS3.dVersion, iCount, stLAS );

            stLogs[iCount] = las.lasFileDataUtility.copyHeader( stLAS, stLogs[iCount] );
            stLogs[iCount] = pCurves[i].getLASData(
                 stLAS3.sDelim, stLAS3.sWrap, stLAS3.dVersion, iCount, stLogs[iCount] );
            stLAS          = las3.las3LoadLogData.mergeLogData( stLogs[iCount], stLAS );

            iCount++;
          }
        }
      }
    }

    if (iCount > 0)
    {
      if (stLogs != null)
      {
        for (i=0; i<iCount; i++)
        {
          if (stLogs[i] != null)
          {
            stLogs[i].delete();
            stLogs[i] = null;
          }
        }
        stLogs = null;
      }
    }

    if (stLAS != null)
    {
	  if (stLAS.iSP > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dSP, stLAS.dNull);
        stLAS.dSPMin = dRange[0];
        stLAS.dSPMax = dRange[1];
	  }

	  if (stLAS.iLIN_1 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_1, stLAS.dNull);
        stLAS.dLIN_1Min = dRange[0];
        stLAS.dLIN_1Max = dRange[1];
	  }

	  if (stLAS.iLIN_2 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_2, stLAS.dNull);
        stLAS.dLIN_2Min = dRange[0];
        stLAS.dLIN_2Max = dRange[1];
	  }

	  if (stLAS.iLIN_3 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_3, stLAS.dNull);
        stLAS.dLIN_3Min = dRange[0];
        stLAS.dLIN_3Max = dRange[1];
	  }

	  if (stLAS.iLIN_4 > -1)
	  {
        double dRange[] = math.mathLAS.computeCurveMinMax(stLAS.dLIN_4, stLAS.dNull);
        stLAS.dLIN_4Min = dRange[0];
        stLAS.dLIN_4Max = dRange[1];
	  }

	  if (stLAS.iLOG_1 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_1, stLAS.dNull);
        stLAS.dLOG_1Min = dRange[0];
        stLAS.dLOG_1Max = dRange[1];
	  }

	  if (stLAS.iLOG_2 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_2, stLAS.dNull);
        stLAS.dLOG_2Min = dRange[0];
        stLAS.dLOG_2Max = dRange[1];
	  }

	  if (stLAS.iLOG_3 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_3, stLAS.dNull);
        stLAS.dLOG_3Min = dRange[0];
        stLAS.dLOG_3Max = dRange[1];
	  }

	  if (stLAS.iLOG_4 > -1)
	  {
        double dRange[] = math.mathLAS.computeLogMinMax(stLAS.dLOG_4, stLAS.dNull);
        stLAS.dLOG_4Min = dRange[0];
        stLAS.dLOG_4Max = dRange[1];
	  }

	  if ((stLAS.iLOG_1 > -1) || (stLAS.iLOG_2 > -1) ||
	      (stLAS.iLOG_3 > -1) || (stLAS.iLOG_4 > -1))
	  {
		dLogMin = 0.1;
		dLogMax = 1000.0;

		if (stLAS.iLOG_1 > -1)
		{
		  if (stLAS.dLOG_1Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_1Min;
		  }

		  if (stLAS.dLOG_1Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_1Max;
		  }
		}

		if (stLAS.iLOG_2 > -1)
		{
		  if (stLAS.dLOG_2Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_2Min;
		  }

		  if (stLAS.dLOG_2Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_2Max;
		  }
		}

		if (stLAS.iLOG_3 > -1)
		{
		  if (stLAS.dLOG_3Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_3Min;
		  }

		  if (stLAS.dLOG_3Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_3Max;
		  }
		}

		if (stLAS.iLOG_4 > -1)
		{
		  if (stLAS.dLOG_4Min < dLogMin)
		  {
		    dLogMin = stLAS.dLOG_4Min;
		  }

		  if (stLAS.dLOG_4Max > dLogMax)
		  {
		    dLogMax = stLAS.dLOG_4Max;
		  }
		}

        stLAS.dLOG_1Min = dLogMin;
        stLAS.dLOG_1Max = dLogMax;


        stLAS.dLOG_2Min = dLogMin;
        stLAS.dLOG_2Max = dLogMax;

        stLAS.dLOG_3Min = dLogMin;
        stLAS.dLOG_3Max = dLogMax;

        stLAS.dLOG_4Min = dLogMin;
        stLAS.dLOG_4Max = dLogMax;
	  }
	}

    return (stLAS);
  }

  /** Method getRockData()
   * <p> This method will retrieve the Rock Data List Structure
   * @return stRock = the Rock Data List Structure
   */

  public rockDataListStruct getRockData()
  {
    int i        = 0;
    int iCount   = 0;
    int iSamples = 0;
    rockDataListStruct stR = null;
    double             dLogMin = 0.0;
    double             dLogMax = 0.0;
    double             data[]  = null;
    double             dRange[] = null;

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _CORE_TYPE)

        {
          if (iCount < iRock)
          {
//			if (pCurves[i].getAction() == cmn.cmnStruct.ON)
//			{
              stRock[iCount] =
                rock.rockDataUtility.copyList( pCurves[i].getRockData() );
//		    }
            iCount++;
          }
        }
      }
    }

    if (iCount == 1)
    {
      stR = rock.rockDataUtility.copyList( stRock[0] );
    }
    else if (iCount > 1)
    {
      for (i=0; i<iCount; i++)
      {
        iSamples = iSamples + stRock[i].iCount;
      }

      if (iSamples > 0)
      {
        stR        = new rockDataListStruct();
        stR.stItem = new rockDataStruct[iSamples];
        stR.iCount = iSamples;

        for (i=0; i<iSamples; i++)
        {
          stR.stItem[i] = new rockDataStruct();
        }
      }

      for (i=0; i<iCount; i++)
      {
        stR = las3.las3LoadCoreData.mergeCoreData( stRock[i], stR );
      }
    }

    if (stR != null)
    {
	  if (stR._LIN_1 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LIN_1 );
        dRange = math.mathLAS.computeCurveMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLIN_1Min = dRange[0];
        stR.dLIN_1Max = dRange[1];
//System.out.println("LINEAR 1 "+ stR.dLIN_1Min +" "+ stR.dLIN_1Max);
	  }

	  if (stR._LIN_2 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LIN_2 );
        dRange = math.mathLAS.computeCurveMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLIN_2Min = dRange[0];
        stR.dLIN_2Max = dRange[1];
//System.out.println("LINEAR 2 "+ stR.dLIN_2Min +" "+ stR.dLIN_2Max);
	  }

	  if (stR._LIN_3 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LIN_3 );
        dRange = math.mathLAS.computeCurveMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLIN_3Min = dRange[0];
        stR.dLIN_3Max = dRange[1];
	  }

	  if (stR._LIN_4 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LIN_4 );
        dRange = math.mathLAS.computeCurveMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLIN_4Min = dRange[0];
        stR.dLIN_4Max = dRange[1];
	  }

	  if (stR._LOG_1 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LOG_1 );
        dRange = math.mathLAS.computeLogMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLOG_1Min = dRange[0];
        stR.dLOG_1Max = dRange[1];
//System.out.println("LOGa 1 "+ stR.dLOG_1Min +" "+ stR.dLOG_1Max);
	  }

	  if (stR._LOG_2 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LOG_2 );
        dRange = math.mathLAS.computeLogMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLOG_2Min = dRange[0];
        stR.dLOG_2Max = dRange[1];
	  }

	  if (stR._LOG_3 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LOG_3 );
        dRange = math.mathLAS.computeLogMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLOG_3Min = dRange[0];
        stR.dLOG_3Max = dRange[1];
	  }

	  if (stR._LOG_4 > -1)
	  {
		data   = stR.getData( rock.rockStandardTools._LOG_4 );
        dRange = math.mathLAS.computeLogMinMax(data, rock.rockDataStruct.dNULL);
        stR.dLOG_4Min = dRange[0];
        stR.dLOG_4Max = dRange[1];
	  }

	  if ((stR._LOG_1 > -1) || (stR._LOG_2 > -1) ||
	      (stR._LOG_3 > -1) || (stR._LOG_4 > -1))
	  {
		dLogMin = 0.1;
		dLogMax = 1000.0;

		if (stR._LOG_1 > -1)
		{
		  if (stR.dLOG_1Min < dLogMin)
		  {
		    dLogMin = stR.dLOG_1Min;
		  }

		  if (stR.dLOG_1Max > dLogMax)
		  {
		    dLogMax = stR.dLOG_1Max;
		  }
		}

		if (stR._LOG_2 > -1)
		{
		  if (stR.dLOG_2Min < dLogMin)
		  {
		    dLogMin = stR.dLOG_2Min;
		  }

		  if (stR.dLOG_2Max > dLogMax)
		  {
		    dLogMax = stR.dLOG_2Max;
		  }
		}

		if (stR._LOG_3 > -1)
		{
		  if (stR.dLOG_3Min < dLogMin)
		  {
		    dLogMin = stR.dLOG_3Min;
		  }

		  if (stR.dLOG_3Max > dLogMax)
		  {
		    dLogMax = stR.dLOG_3Max;
		  }
		}

		if (stR._LOG_4 > -1)
		{
		  if (stR.dLOG_4Min < dLogMin)
		  {
		    dLogMin = stR.dLOG_4Min;
		  }

		  if (stR.dLOG_4Max > dLogMax)
		  {
		    dLogMax = stR.dLOG_4Max;
		  }
		}

        stR.dLOG_1Min = dLogMin;
        stR.dLOG_1Max = dLogMax;


        stR.dLOG_2Min = dLogMin;
        stR.dLOG_2Max = dLogMax;

        stR.dLOG_3Min = dLogMin;
        stR.dLOG_3Max = dLogMax;

        stR.dLOG_4Min = dLogMin;
        stR.dLOG_4Max = dLogMax;
	  }
    }

    if (iCount > 0)
    {
      if (stRock != null)
      {
        for (i=0; i<iCount; i++)
        {
          if (stRock[i] != null)
          {
            stRock[i].delete();
            stRock[i] = null;
          }
        }
        stRock = null;
      }
    }

    return (stR);
  }

  /** Method getTops()
   * <p> This method will retrieve the Stratigraphic Units Data Structure
   * @return st = the Stratigraphic Units Data List Structure
   */

  public stratListStruct getTops()
  {
    int i        = 0;
    int iCount   = 0;
    int iSamples = 0;
    stratListStruct stR = null;

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _TOPS_TYPE)

        {
          if (iCount < iTops)
          {
            stTops[iCount] =
              horizon.strat.stratUtility.copyList( pCurves[i].getTops() );
            iCount++;
          }
        }
      }
    }

    if (iCount == 1)
    {
      stR = horizon.strat.stratUtility.copyList( stTops[0] );
    }
    else if (iCount > 1)
    {
      for (i=0; i<iCount; i++)
      {
        if (stTops[i] != null)
          iSamples = iSamples + stTops[i].iCount;
      }

      if (iSamples > 0)
      {
        stR        = new stratListStruct();
        stR.stItem = new stratStruct[iSamples];
        stR.iCount = iSamples;

        for (i=0; i<iSamples; i++)
        {
          stR.stItem[i] = new stratStruct();
          stR.stItem[i].depthStart = stLAS.dNull;
        }
      }

      for (i=0; i<iCount; i++)
      {
        if (stTops[i] != null)
        {
          stR = las3.las3LoadTopsData.mergeTopsData( stLAS.dNull, stTops[i], stR );
	    }
      }
    }

    if (iCount > 0)
    {
      if (stTops != null)
      {
        for (i=0; i<iCount; i++)
        {
          if (stTops[i] != null)
          {
            stTops[i].delete();
            stTops[i] = null;
          }
        }
        stTops = null;
      }
    }

    return (stR);
  }

  /** Method getControl()
   * <p> This method will retrieve the Control Data Structure
   * @return st = the Control Data Structure
   */

  public iqstratControlStruct getControl()
  {
    int i        = 0;
    int iCount   = 0;
    int iSamples = 0;
    iqstratControlStruct stR = null;

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _CNTRL_TYPE)

        {
          if (iCount < iControl)
          {
            stControl[iCount] =
              iqstrat.iqstratControlUtility.copy( pCurves[i].getControl() );
            iCount++;
          }
        }
      }
    }

    if (stControl != null)
    {
      if (stControl[0] != null)
      {
        stR = iqstrat.iqstratControlUtility.copy( stControl[0] );
        stControl[0].delete();
        stControl[0] = null;
      }
      stControl = null;
    }

    return (stR);
  }

  /** Method getRemarks()
   * <p> This method will retrieve the Remarks Data List Structure
   * @return st = the Remarks Data List Structure
   */

  public iqstratRemarkListStruct getRemarks()
  {
    int i        = 0;
    int iCount   = 0;
    int iSamples = 0;
    iqstratRemarkListStruct stR = null;

    for (i=0; i<iTotal; i++)
    {
      if (pCurves[i] != null) // Curve Selection Panel
      {
        if (pCurves[i].getDataType() == _GEO_TYPE)

        {
          if (iCount < iRemarks)
          {
            stRemarks[iCount] =
              iqstrat.iqstratRemarkUtility.copyList( pCurves[i].getRemarks() );
            iCount++;
          }
        }
      }
    }

    if (stRemarks != null)
    {
      if (stRemarks[0] != null)
      {
        stR = iqstrat.iqstratRemarkUtility.copyList( stRemarks[0] );
        stRemarks[0].delete();
        stRemarks[0] = null;
      }
      stRemarks = null;
    }

    return (stR);
  }

  /* ===================================================================== *
   * ---------------------------- COMPUTE -------------------------------- *
   * ===================================================================== */

  /** Method computeDepths()
   * <p> This method will compute the initial depth information for the
   *     new LAS File
   * @param  iCount = total number of LAS File Data Structures
   * @return st     = LAS File Data Structure
   */

  private lasFileDataStruct computeDepths( int iCount )
  {
    lasFileDataStruct st = new lasFileDataStruct();
    int     i,j;
    double  dTemp      = 0.0;
    int     iSteps     = 0;
    int     iStart[]   = null;
    double  dStart[]   = null;
    double  dStep[]    = null;
    int     iStepVal[] = null;
    // Allowed Step Size  1.0, 0.5, 0.1, other
    int     iValue[]   = { 0,   0,   0,  0 };

    int     iDepth     = 0;
    double  diff       = 0.0;

    if (iCount > 0)
    {
      iStart   = new int[iCount];
      dStart   = new double[iCount];
      dStep    = new double[iCount];
      iStepVal = new int[iCount];

      for (i=0; i<iCount; i++)
      {
        iStart[i]   = 0;
        dStart[i]   = 0.0;
        dStep[i]    = 0.0;
        iStepVal[i] = _SAME;

        if (stLogs[i].depthStart > stLogs[i].depthEnd)
        {
          dTemp                = stLogs[i].depthStart;
          stLogs[i].depthStart = stLogs[i].depthEnd;
          stLogs[i].depthEnd   = dTemp;

          stLogs[i].depthStep  = Math.abs(stLogs[i].depthStep);
        }

        dStart[i] = stLogs[i].depthStart;
        iStart[i] = (int) dStart[i];
        dStart[i] = dStart[i] - (double) iStart[i];
        dStep[i]  = stLogs[i].depthStep;

        if (dStep[i] == 1.0)      iValue[0]++;
        else if (dStep[i] == 0.5) iValue[1]++;
        else if (dStep[i] == 0.1) iValue[2]++;
        else                      iValue[3]++;

        if (i==0)
        {
          st.depthStart = stLogs[0].depthStart;
          st.depthEnd   = stLogs[0].depthEnd;
        }

        dTemp  = stLogs[i].depthStart;
        if (dTemp < st.depthStart) st.depthStart = dTemp;

        dTemp  = stLogs[i].depthEnd;
        if (dTemp > st.depthEnd) st.depthEnd = dTemp;
      }
    }

    st.depthStep = dStep[0];

    if (iValue[3] > 0)
    {
      for (i=0; i<iCount; i++)
      {
        if (dStep[i] < 1.0) st.depthStep = 1.0;
        if (dStep[i] < 0.5) st.depthStep = 0.5;
        if (dStep[i] < 0.1) st.depthStep = 0.1;
      }
    }

    if ((iValue[0] > 0) && (iValue[1] > 0))
      st.depthStep = 1.0;

    if ((iValue[0] > 0) && (iValue[2] > 0))
      st.depthStep = 1.0;

    if ((iValue[1] > 0) && (iValue[2] > 0))
      st.depthStep = 0.5;

    iDepth = (int) st.depthStart * 10;
    diff   = (st.depthStart * 10.0 - (double) iDepth) / 10.0;

    if (st.depthStart < 0.0) st.depthStart = 0.0;
    else if (diff != 0.0)    st.depthStart = (double) iDepth;

    st.iRows = 1 + (int)( Math.abs(st.depthEnd-st.depthStart) / st.depthStep );

    return (st);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    int i=0;

    // Input Variables

    stStruct  = null; // Global IQSTRAT Data Structure
    stLAS3    = null; // LAS 3 File Data List Structure
    stLAS     = null; // LAS File Data Structure

    if (stLogs != null)
    {
      for (i=0; i<iLogs; i++)
      {
        if (stLogs[i] != null)
          stLogs[i].delete();
        stLogs[i] = null;
      }
    }
    stLogs = null;
    iLogs  = 0;

    if (stRock != null)
    {
      for (i=0; i<iRock; i++)
      {
        if (stRock[i] != null)
          stRock[i].delete();
        stRock[i] = null; // Rock Data List Structure
      }
    }
    stRock = null;
    iRock  = 0;

    if (stTops != null)
    {
      for (i=0; i<iTops; i++)
      {
        if (stTops[i] != null)
          stTops[i].delete();
        stTops[i] = null;
      }
    }
    stTops = null;
    iTops  = 0;

    if (stControl != null)
    {
      for (i=0; i<iControl; i++)
      {
        if (stControl[i] != null)
          stControl[i].delete();
        stControl[i] = null;
      }
    }
    stControl = null;
    iControl  = 0;

    if (stRemarks != null)
    {
      for (i=0; i<iRemarks; i++)
      {
        if (stRemarks[i] != null)
          stRemarks[i].delete();
        stRemarks[i] = null;
      }
    }
    stRemarks = null;
    iRemarks  = 0;

    // Global Variables

    sReadMethod = null;  // File Read Method
    sDirectory  = null;  // Directory Path
    sFilename   = null;  // File name of LAS File

    for (i=0; i<iTotal; i++)
      pCurves[i] = null;   // Curve Selection Panel

    sCurves = null;
    iData   = null;   // LAS 3 Order, Type, Array
    iTotal  = 0;      // Total Number of Panels
  }
}

/*
 *  @version 1.1 03/05/2013
 *  @author  John Victorine
 */
