/*
 * @las3CurvesLog.java Version 1.1 03/05/2013
 *
 * Copyright (c) 2013 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las3;

import java.awt.*;

import cmn.cmnStruct;                   // Global Common Data Structure
import cmn.cmnLASCurveListStruct;       // Common LAS Curve Data List Structure
import cmn.cmnLASCurveStruct;           // Common LAS Curve Data Structure

import iqstrat.iqstratStruct;           // Global IQSTRAT Data Structure
import iqstrat.iqstratControlStruct;    // Plot Control Data Structure
import iqstrat.iqstratRemarkListStruct; // Remarks Data List Structure

import horizon.strat.stratListStruct;   // Stratigraphic Units List Structure

import rock.rockDataListStruct;         // Rock Data List Structure

import las3.las3Struct;                 // LAS 3 Data Sturcture
import las.lasFileDataStruct;           // LAS File Data Structure
import las3.io.las3ReadFile;            // Open, Read & Parse LAS Files

/** Class las3CurvesLog
 *  <p> This Class will display the Curves sections of the LAS 3 File and allow
 *      the user to select the curves they wish to display on the KGS web apps
 *      and to identify unknown curves with KGS Standard Curves.
 *
 *  @version 1.1 03/05/2013
 *  @author  John R. Victorine
 */

public class las3CurvesLog
{
  // Input Variables

  private iqstratStruct     stStruct = null;  // Global IQSTRAT Data Structure
  private cmnStruct         stCMN    = null;  // Common Data Structure
  private las3Struct        stLAS3   = null;  // LAS 3 File Data Structure
  private lasFileDataStruct stLAS    = null;  // LAS File Data Structure
  private double            dVersion = 0.0;   // Version Number of LAS File
  private int               iqstrato = -1;    // Profile Saved Data with type

  // Global Variables

  private static final int OFF  = cmn.cmnStruct.OFF;
  private static final int ON   = cmn.cmnStruct.ON;

  private int    iCurve = -1;

  private int    iArray = 0;                       // Array Number
  private int    iType  = las3.las3Constants._NONE; // Type of Data Identifier
  private String sType  = "";                      // Type of Data Name Set
  private String sLabel = "";

  private int iTotal      = 0;    // Total Number of Log Curves

  private int iStandard   = 0;    // Total Number Standard Curves
  private int iSelected[] = null; // Array of Selected Standard Curves

  private cmnLASCurveListStruct stCurves = null; // LAS File Curves
//  private lasToolsFrame         pTools   = null; // Standard Tools List Frame

  public static final int _IGNORE        = 0;
  public static final int _CHECK         = 1;

  public static final int _LAS_TYPE      = las.gui.lasToolsTable._LAS_TYPE;
  public static final int _CORE_TYPE     = las.gui.lasToolsTable._CORE_TYPE;
  public static final int _TOPS_TYPE     = las.gui.lasToolsTable._TOPS_TYPE;
  public static final int _CNTRL_TYPE    = las.gui.lasToolsTable._CNTRL_TYPE;
  public static final int _GEO_TYPE      = las.gui.lasToolsTable._GEO_TYPE;

  private int iData = _LAS_TYPE;   // Type of Standard Tools List to Use

  public static final int _NONE          = las3.las3Constants._NONE;

  public static final int _VERSION       = las3.las3Constants._VERSION;
  public static final int _WELL          = las3.las3Constants._WELL;
  public static final int _LOG           = las3.las3Constants._LOG;
  public static final int _CORE          = las3.las3Constants._CORE;
  public static final int _DRILLING      = las3.las3Constants._DRILLING;
  public static final int _INCLINOMETRY  = las3.las3Constants._INCLINOMETRY;
  public static final int _TOPS          = las3.las3Constants._TOPS;
  public static final int _TEST          = las3.las3Constants._TEST;
  public static final int _PERFORATION   = las3.las3Constants._PERFORATION;
  public static final int _IQ_CONTROL    = las3.las3Constants._IQ_CONTROL;
  public static final int _IQ_LOG        = las3.las3Constants._IQ_LOG;
  public static final int _IQ_CORE       = las3.las3Constants._IQ_CORE;
  public static final int _IQ_SEQ_STRAT  = las3.las3Constants._IQ_SEQ_STRAT;
  public static final int _IQ_FLOW       = las3.las3Constants._IQ_FLOW;
  public static final int _IQ_GEO_REPORT = las3.las3Constants._IQ_GEO_REPORT;
  public static final int _IQ_SHALE      = las3.las3Constants._IQ_SHALE;
  public static final int _IQ_IMAGES     = las3.las3Constants._IQ_IMAGES;
  public static final int _IQ_BRINE      = las3.las3Constants._IQ_BRINE;
  public static final int _IQ_BIO        = las3.las3Constants._IQ_BIO;
  public static final int _IQ_ENV        = las3.las3Constants._IQ_ENV;

  public static final int _MNEMONIC      = las3.las3Struct._MNEMONIC;
  public static final int _UNIT          = las3.las3Struct._UNIT;
  public static final int _VALUE         = las3.las3Struct._VALUE;
  public static final int _DESCRIPTION   = las3.las3Struct._DESCRIPTION;
  public static final int _FORMAT        = las3.las3Struct._FORMAT;
  public static final int _ASSOCIATION   = las3.las3Struct._ASSOCIATION;

  /** Constructor las3CurvesLog()
   *  <p> This is the Constructor for this class.
   * @param stStruct = Global IQSTRAT Data Structure
   * @param stLAS    = LAS File Data Structure
   * @param dVersion = The Version Number of LAS File
   * @param iqstrato = indicator that the data type has a profile saved data
   * @param stLAS3   = LAS 3 File Data Structure
   */

  public las3CurvesLog( iqstratStruct     stStruct,
                        lasFileDataStruct stLAS,
                        double            dVersion,
                        int               iqstrato,
                        las3Struct        stLAS3 )
  {
    try
    {
      this.stStruct = stStruct;
      this.stCMN    = stStruct.stCMN;
      this.stLAS    = stLAS;
      this.stLAS3   = stLAS3;
      this.dVersion = dVersion;
      this.iqstrato = iqstrato;

      if (stLAS3 != null)
      {
        if (stLAS3.iqstrat > -1) this.iqstrato = -1;
      }

      loadData( las3.las3Utility.copy( stLAS3 ) );  // Load Data Structure

      jbInit();

      // Set Widgets

      setDefinition();   // Build & Display ~{}_Definitions Section Text
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    iTotal       = 0;

    if (stCurves != null)
    {
      iTotal = stCurves.iCount;

      switch (iData)
      {
        case _LAS_TYPE:
          iStandard = las.lasStandardTools.TOTAL;
          break;
        case _CORE_TYPE:
          iStandard = rock.rockStandardTools.TOTAL_DATA;
          break;
        case _TOPS_TYPE:
          iStandard = horizon.strat.stratStandardTools.TOTAL;
          break;
        case _CNTRL_TYPE:
          iStandard = iqstrat.iqstratControlStandardTools.TOTAL;
          break;
        case _GEO_TYPE:
          iStandard = iqstrat.iqstratRemarkStandardTools.TOTAL;
          break;
      }

      iSelected = new int[iStandard];
      for (int j=0; j<iStandard; j++)
        iSelected[j] = -1;
    }
  }

  /** Method loadData()
   * <p> This method will load the data structure
   * @param st = LAS 3 Data Structure
   */

  private void loadData( las3Struct st )
  {
    if (st != null)
    {
      iArray = st.iArray;              // Array Number
      iType  = st.iType;               // Type of Data Identifier
      sType  = new String( st.sType ); // Type of Data Name Set

      switch (st.iType)
      {
        case _NONE:
        case _VERSION:       // Version Informaiton
        case _WELL:          // Well Information
          break;
        case _LOG:           // Log Data Section
          stCurves = las3.las3Map.getLASCurves( st );
          stCurves = las3.las3Map.mapToKGS(
                         cmn.cmnStruct.DOE_CO2_DATA + "/las_standard_tools.xml", stCurves );
//                         stCMN.sPath + "/las_standard_tools.xml", stCurves );
          iData = _LAS_TYPE;
          break;
        case _CORE:          // Core Data Section
          stCurves = las3.las3Map.getLASCurves( st );
          stCurves = las3.las3Map.mapToKGS( _CORE, stCurves );
          iData    = _CORE_TYPE;
          break;
        case _TOPS:          // Tops Data Section
          stCurves = las3.las3Map.getLASCurves( st );
          stCurves = las3.las3Map.mapToKGS( _TOPS, stCurves );
          iData = _TOPS_TYPE;
          break;
        case _IQ_CONTROL:
          stCurves = las3.las3Map.getLASCurves( st );
          stCurves = las3.las3Map.mapToKGS( _IQ_CONTROL, stCurves );
          iData    = _CNTRL_TYPE;
          break;
        case _IQ_GEO_REPORT:
          stCurves = las3.las3Map.getLASCurves( st );
          stCurves = las3.las3Map.mapToKGS( _IQ_GEO_REPORT, stCurves );
          iData    = _GEO_TYPE;
          break;
      }
    }
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getArray()
   * <p> This method will get the Array Number of the LAS 3 Data Type
   * @return iArray = the Array Number of the LAS 3 Data Type
   */

  public int getArray() { return (iArray); }

  /** Method getTypeID()
   * <p> This method will get the LAS 3 Data Type ID
   * @return iType = the LAS 3 Data Type ID
   */

  public int  getTypeID() { return (iType); }

  /** Method getType()
   * <p> This method will get the LAS 3 Data Type
   * @return iType = the LAS 3 Data Type
   */

  public String getType() { return (sType); }

  /** Method getLabel()
   * <p> This method will get the label for the tabbed Pane
   * @return sLabel = the label for the tabbed Pane
   */

  public String getLabel() { return (sLabel); }

  /** Method getDataType()
   * <p> This method will return the data type
   * @return iData = Data Type
   */

  public int getDataType() { return (iData); }

  /** Method getLAS3()
   * <p> This method will pass the LAS 3 File data structure with the selected
   *     curves and mnemonics.
   * @return stLAS3 = modified LAS 3 File data structure
   *

  public las3Struct getLAS3()
  {
    int i,j;
    String sTemp = "";

    if (stLAS3 != null)
    {
      stLAS3.iSelectRows = stLAS3.iCurveRows;
      stLAS3.select      = new String[stLAS3.iCurveRows][las3.las3Struct._TOTAL];

      for (i=0; i<stLAS3.iCurveRows; i++)
      {
        for (j=0; j<las3.las3Struct._TOTAL; j++)
          stLAS3.select[i][j] = new String("");

        if (stLAS3.select != null)
        {
          stLAS3.select[i][_MNEMONIC] = new String( btnMnem[i].getText() );
          sTemp = new String( removeChar( lblUnit[i].getText(), '.' ).trim() );
          stLAS3.select[i][_UNIT]  = new String( sTemp );
          stLAS3.select[i][_VALUE] = new String("");
          sTemp = new String( removeChar( lblDesc[i].getText(), ':' ).trim() );
          stLAS3.select[i][_DESCRIPTION] = new String( sTemp );
        }
      }
    }

    return (stLAS3);
  }
*/
  /** Method getLAS()
   * <p> This method will retrieve the LAS File Data Structure
   * @param  sDelim   = Delimiter used to separate data values
   * @param  sWrap    = Is the data line wrapped ( For LAS 2.0 Files )
   * @param  dVersion = Version number of LAS File
   * @param  iLog     = Log Number
   * @param  st       = the LAS File Data Structure
   * @return st       = the LAS File Data Structure
   */

  public lasFileDataStruct getLAS( double dVersion, int iLog, lasFileDataStruct st )
  {
    int i,j;
    int iSum  = 1;
    int iCnt  = 0;
    int iTool = -1;
    cmnLASCurveListStruct stNew = null;
    lasFileDataStruct     stC   = null;
    String sKIDo   = "0";
    String sKID    = "0";
    String sData[] = null;
    int    iFound  = -1;
    int    iType   = -1;

    if (iData == _LAS_TYPE)
    {
      if (dVersion == 2.0)
      {
        // LAS 3 File Parameters

        st.iParamRows = stLAS3.iParamRows;    // Total number of rows
        st.iParamCols = stLAS3.iParamCols;    // Total number of columns
        st.sParams    = new String[st.iParamRows][st.iParamCols];

        for (i=0; i<st.iParamRows; i++)
        {
          for (j=0; j<st.iParamCols; j++)
          {
            // Array holding the Parameter Definitions
            st.sParams[i][j] = new String( stLAS3.sParams[i][j] );
          }
        }
      }

      // Load the SELECTED Curves Settings into the LAS 3 structure

//      getLAS3();

      // Copy the LAS 3 Curves & Parameters to the LAS File Data Structure

      st.stLAS3[iLog] = las3.las3Utility.copyWOData( stLAS3 );
    }

    return (st);
  }

  /** Method getLASData()
   * <p> This method will retrieve the LAS File Data Structure
   * @param  sDelim   = Delimiter used to separate data values
   * @param  sWrap    = Is the data line wrapped ( For LAS 2.0 Files )
   * @param  dVersion = Version number of LAS File
   * @param  iLog     = Log Number
   * @param  st       = the LAS File Data Structure
   * @return st       = the LAS File Data Structure
   */

  public lasFileDataStruct getLASData( String sDelim, String sWrap, double dVersion,
                                       int iLog, lasFileDataStruct st )
  {
    int i,j,k;
    int iSum  = 1;
    int iCnt  = 0;
    int iTool = -1;
    cmnLASCurveListStruct stNew = null;
    lasFileDataStruct     stC   = null;
//    lasFileDataStruct     st    = null;
    String sKIDo   = "0";
    String sKID    = "0";
    String sData[] = null;
    int    iFound  = -1;
    int    iType   = -1;
    double data    = 0.0;
    int    iStart  = 0;
    double dStart  = 0.0;
    double dEnd    = 0.0;
    double dStep   = 0.0;
    int    iStep   = 0;
    int    iRows   = 0;
    las3ReadFile p3 = null;
    int    iCont   = 0;

    if (iData == _LAS_TYPE)
    {
      for (j=0; j<iStandard; j++)
      {
        if (iSelected[j] > -1)
        {
          iSum++;
        }
      }

      if (stCurves != null)
      {
        if (stCurves.iCount > 0)
        {
          stNew          = new cmnLASCurveListStruct();
          stNew.iCount   = iSum;
          stNew.stItem   = new cmnLASCurveStruct[iSum];

          for (i=0; i<stCurves.iCount; i++)
          {
            sKID   = stCurves.stItem[i].sToolKID;

            iTool = -1;
            for (j=0; j<iStandard; j++)
            {
              if (iSelected[j] > -1)
              {
                sKIDo = las.lasStandardTools.LAS_TOOLS[j][0];
                if (sKID.equals(sKIDo))
                {
                  iTool = j;

  				  if (iTool > -1)
  				  {
				    for (k=0; k<iCnt; k++)
				    {
				      if (sKIDo.equals(stNew.stItem[k].sToolKID))
				      {
				    	iTool = -1;
				      }
			        }
                  }
				}
              }
            }

            if ((iTool > -1) || (i==0))
            {
              if (iCnt < iSum)
              {
                stCurves.stItem[i].iCurveNumber = i; // JRV 11/28/2011
                stNew.stItem[iCnt] =
                    cmn.cmnLASCurveUtility.copy( stCurves.stItem[i] );
                iCnt++;
              }
            }
          }
        }
      }

      // Get the LAS Data Structure

      stC   = new lasFileDataStruct();
      stC   = las.lasFileDataUtility.findCurves( stNew, stC );
      stLAS = las.lasFileDataUtility.findCurves( stNew, stLAS );
      st    = las.lasFileDataUtility.findCurves( stNew, st );

      // Get Log Data from LAS File

      iType = util.utilFileIO.DIRECTORY;
      if (st.sReadMethod.equals("URL"))
        iType = util.utilFileIO.URL_FILE;

      // Open and compute the depth Range & Step

      p3 = new las3ReadFile();
      p3.Open( iType, st.sDirectory, st.sFilename );        // Open LAS File
      iFound = p3.ReadFile( las3.las3Constants._LOG, iLog, dVersion );

      if (iFound == 1)        // IF Log Section is Found
      {
        while (iFound == 1) // While Data line is not null
        {
          sData = p3.getNextLine(sDelim, sWrap, dVersion, stLAS3.iColumns);
          if (sData == null)
            iFound = -1;
          else
          {
            data = las3.las3LoadLogData.stringToDouble( sData[0], stLAS.dNull );

            if (data != stLAS.dNull)
            {
			  if (iStart == 0)
			  {
			    dStart = data;
			    dEnd   = data;

			    sData  = p3.getNextLine(sDelim, sWrap, dVersion, stLAS3.iColumns);
			    if (sData != null)
			    {
                  data  = las3.las3LoadLogData.stringToDouble( sData[0], stLAS.dNull );
                  iStep = (int) ( Math.abs( data - dStart ) * 100.0 );
                  dStep = (double) iStep / 100.0;
			    }

			    iStart = 1;
			  }

			  if (dEnd < data)
			    dEnd = data;
			}
		  }
        }
      }

      // Compute the Data Structrue

      iRows = (int) ( Math.abs(dEnd - dStart) / dStep ) + 1;

      st.depthStart = dStart;
      st.depthEnd   = dEnd;
      st.depthStep  = dStep;

      st = las3.las3LoadLogData.initData( iRows, st );

      p3.Close();                                           // Close LAS File

      if (p3 != null) // Close & clear memory for the LAS 3 File Read Class
        p3.delete();
      p3 = null;

      // Now Add the Log Data to the Data Structure
//System.out.println(iRows+" "+dStart+" "+dEnd+" "+dStep);

      p3 = new las3ReadFile();
      p3.Open( iType, st.sDirectory, st.sFilename );        // Open LAS File
      iFound = p3.ReadFile( las3.las3Constants._LOG, iLog, dVersion );

      if (iFound == 1)        // IF Log Section is Found
      {
        while (iFound == 1) // While Data line is not null
        {
          sData = p3.getNextLine(sDelim, sWrap, dVersion, stLAS3.iColumns);
          if (sData == null) iFound = -1;
          st = las3.las3LoadLogData.addData( sData, stLAS3.iColumns, stC, st );
        }
      }

      p3.Close();                                           // Close LAS File
      stC.delete();
      stC = null;

      // Deallocate Java Memory

      if (p3 != null) // Close & clear memory for the LAS 3 File Read Class
        p3.delete();
      p3 = null;

      // Delete the Common LAS Curves Data List Structure

      if (stNew != null)
        stNew.delete();
      stNew = null;
    }

    return (st);
  }

  /** Method getRockData()
   * <p> This method will retrieve the Rock Data List Structure
   * @return st = the Rock Data List Structure
   */

  public rockDataListStruct getRockData()
  {
    int i,j;
    int iSum  = 1;
    int iCnt  = 0;
    int iTool = -1;
    cmnLASCurveListStruct stNew = null;
    rockDataListStruct    st    = new rockDataListStruct();
    String sKIDo  = "0";
    String sKID   = "0";

    if (iData == _CORE_TYPE)
    {
      for (j=0; j<iStandard; j++)
      {
        if (iSelected[j] > -1)
        {
          iSum++;
        }
      }

      if (stCurves != null)
      {
        if (stCurves.iCount > 0)
        {
          stNew          = new cmnLASCurveListStruct();
          stNew.iCount   = iSum;
          stNew.stItem   = new cmnLASCurveStruct[iSum];

          for (i=0; i<stCurves.iCount; i++)
          {
            sKID   = stCurves.stItem[i].sToolKID;

            iTool = -1;
            for (j=0; j<iStandard; j++)
            {
              if (iSelected[j] > -1)
              {
                sKIDo = rock.rockStandardTools.ROCK_TOOLS[j][6];
                if (sKID.equals(sKIDo))
                  iTool = j;
              }
            }

            if (iTool > -1)
            {
              if (iCnt < iSum)
              {
                stNew.stItem[iCnt] =
                    cmn.cmnLASCurveUtility.copy( stCurves.stItem[i] );
                iCnt++;
              }
            }
          }

          stNew.iCount   = iCnt;
        }
      }

      // Get the LAS Data Structure

      st = las3.las3LoadCoreData.getData( stLAS3 );
      st = las3.las3LoadCoreData.loadData( stLAS.dNull, stLAS3, stNew, st );

      if (dVersion == 2.0)
      {
        // LAS 3 File Parameters

        st.iParamRows = stLAS3.iParamRows;    // Total number of rows
        st.iParamCols = stLAS3.iParamCols;    // Total number of columns
        st.sParams    = new String[st.iParamRows][st.iParamCols];

        for (i=0; i<st.iParamRows; i++)
        {
          for (j=0; j<st.iParamCols; j++)
          {
            // Array holding the Parameter Definitions
            st.sParams[i][j] = new String( stLAS3.sParams[i][j] );
          }
        }
      }

//      getLAS3(); // Load the SELECTED Curves Settings into the LAS 3 structure

      // Copy the LAS 3 Curves & Parameters to the LAS File Data Structure

      st.stLAS3 = new las3Struct[1];
      st.iLogs  = 1;
      st.stLAS3[0] = las3.las3Utility.copyWOData( stLAS3 );

      // Delete the Common LAS Curves Data List Structure

      if (stNew != null)
        stNew.delete();
      stNew = null;
    }

    return (st);
  }

  /** Method getTops()
   * <p> This method will retrieve the Stratigraphic Units Data Structure
   * @return st = the Stratigraphic Units Data List Structure
   */

  public stratListStruct getTops()
  {
    int i,j;
    int iSum  = 1;
    int iCnt  = 0;
    int iTool = -1;
    cmnLASCurveListStruct stNew = null;
    stratListStruct       st    = null;
    String sKIDo = "0";
    String sKID  = "0";

    if (iData == _TOPS_TYPE)
    {
      for (j=0; j<iStandard; j++)
      {
        if (iSelected[j] > -1)
        {
          iSum++;
        }
      }

      if (stCurves != null)
      {
        if (stCurves.iCount > 0)
        {
          stNew          = new cmnLASCurveListStruct();
          stNew.iCount   = iSum;
          stNew.stItem   = new cmnLASCurveStruct[iSum];

          for (i=0; i<stCurves.iCount; i++)
          {
            sKID   = stCurves.stItem[i].sToolKID;
            iTool = -1;
            for (j=0; j<iStandard; j++)
            {
              if (iSelected[j] > -1)
              {
                sKIDo = horizon.strat.stratStandardTools.CURVES[j][0];
                if (sKID.equals(sKIDo))
                {
                  iTool = j;
                }
              }
            }

            if (iTool > -1)
            {
              if (iCnt < iSum)
              {
                stNew.stItem[iCnt] =
                    cmn.cmnLASCurveUtility.copy( stCurves.stItem[i] );
                iCnt++;
              }
            }
          }

          stNew.iCount   = iCnt;
        }
      }

      // Get the Stratigraphic Units Data List Structure

      st = las3.las3LoadTopsData.getData( stLAS3 );
      st = las3.las3LoadTopsData.loadData( stLAS.dNull, stLAS3, stNew, st );

      if (dVersion == 2.0)
      {
        // LAS 3 File Parameters

        st.iParamRows = stLAS3.iParamRows;    // Total number of rows
        st.iParamCols = stLAS3.iParamCols;    // Total number of columns
        st.sParams    = new String[st.iParamRows][st.iParamCols];

        for (i=0; i<st.iParamRows; i++)
        {
          for (j=0; j<st.iParamCols; j++)
          {
            // Array holding the Parameter Definitions
            st.sParams[i][j] = new String( stLAS3.sParams[i][j] );
          }
        }
      }

      // Delete the Common LAS Curves Data List Structure

      if (stNew != null)
        stNew.delete();
      stNew = null;
    }

    return (st);
  }

  /** Method getControl()
   * <p> This method will retrieve the Control Data Structure
   * @return st = the Control Data Structure
   */

  public iqstratControlStruct getControl()
  {
    int i,j;
    int                   iSum  = 1;
    int                   iCnt  = 0;
    int                   iTool = -1;
    cmnLASCurveListStruct stNew = null;
    iqstratControlStruct  st    = null;
    String                sKIDo = "0";
    String                sKID  = "0";

    if (iData == _CNTRL_TYPE)
    {
      for (j=0; j<iStandard; j++)
      {
        if (iSelected[j] > -1)
        {
          iSum++;
        }
      }

      if (stCurves != null)
      {
        if (stCurves.iCount > 0)
        {
          stNew          = new cmnLASCurveListStruct();
          stNew.iCount   = iSum;
          stNew.stItem   = new cmnLASCurveStruct[iSum];

          for (i=0; i<stCurves.iCount; i++)
          {
            sKID   = stCurves.stItem[i].sToolKID;

            iTool = -1;
            for (j=0; j<iStandard; j++)
            {
              if (iSelected[j] > -1)
              {
                sKIDo = iqstrat.iqstratControlStandardTools.CURVES[j][0];
                if (sKID.equals(sKIDo))
                {
                  iTool = j;
                }
              }
            }

            if (iTool > -1)
            {
              if (iCnt < iSum)
              {
                stNew.stItem[iCnt] =
                    cmn.cmnLASCurveUtility.copy( stCurves.stItem[i] );
                iCnt++;
              }
            }
          }

          stNew.iCount   = iCnt;
        }
      }

      // Get the Control Data Structure

      st = las3.las3LoadControlData.getData( stLAS3 );
      st = las3.las3LoadControlData.loadData( stLAS.dNull, stLAS3, stNew, st );

      // Delete the Common LAS Curves Data List Structure

      if (stNew != null)
        stNew.delete();
      stNew = null;
    }

    return (st);
  }

  /** Method getRemarks()
   * <p> This method will retrieve the Remarks Data List Structure
   * @return st = the Remarks Data List Structure
   */

  public iqstratRemarkListStruct getRemarks()
  {
    int i,j;
    int iSum  = 1;
    int iCnt  = 0;
    int iTool = -1;
    cmnLASCurveListStruct   stNew = null;
    iqstratRemarkListStruct st    = null;
    String sKIDo = "0";
    String sKID  = "0";

    if (iData == _GEO_TYPE)
    {
      for (j=0; j<iStandard; j++)
      {
        if (iSelected[j] > -1)
        {
          iSum++;
        }
      }

      if (stCurves != null)
      {
        if (stCurves.iCount > 0)
        {
          stNew          = new cmnLASCurveListStruct();
          stNew.iCount   = iSum;
          stNew.stItem   = new cmnLASCurveStruct[iSum];

          for (i=0; i<stCurves.iCount; i++)
          {
            sKID   = stCurves.stItem[i].sToolKID;

            iTool = -1;
            for (j=0; j<iStandard; j++)
            {
              if (iSelected[j] > -1)
              {
                sKIDo = iqstrat.iqstratRemarkStandardTools.CURVES[j][0];
                if (sKID.equals(sKIDo))
                {
                  iTool = j;
                }
              }
            }

            if (iTool > -1)
            {
              if (iCnt < iSum)
              {
                stNew.stItem[iCnt] =
                    cmn.cmnLASCurveUtility.copy( stCurves.stItem[i] );
                iCnt++;
              }
            }
          }

          stNew.iCount   = iCnt;
        }
      }

      // Get the Perforations Data List Structure

      st = las3.las3LoadRemarksData.getData( stLAS3 );
      st = las3.las3LoadRemarksData.loadData( stLAS.dNull, stLAS3, stNew, st );

      // Delete the Common LAS Curves Data List Structure

      if (stNew != null)
        stNew.delete();
      stNew = null;
    }

    return (st);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setDefinition()
   * <p> This method will set the Curve Definition section text area
   */

  private void setDefinition()
  {
    int    iTool  = -1;
    String str    = "";
    String sKIDo  = "0";
    String sKID   = "0";
    String sMnem  = "";
    String sUnit  = "";
    String sDesc  = "";
    String sAssoc = "";

    if (stCurves != null)
    {
      for (int j=0; j<iStandard; j++)
        iSelected[j] = -1;
    }

    for (int i=0; i<iTotal; i++)
    {
      if (stCurves != null)
      {
        sKID   = stCurves.stItem[i].sToolKID;
        sMnem  = stCurves.stItem[i].sMnemonic;
        sUnit  = stCurves.stItem[i].sCurveUnits;
        sDesc  = stCurves.stItem[i].sCurveDescription;

        sAssoc = "";
        if (stCurves.stItem[i].iAssoc > 0)
        {
          sAssoc = new String("| ");
          for (int j=0; j<stCurves.stItem[i].iAssoc; j++)
          {
            if (j>0)
              sAssoc = new String(sAssoc + ", ");
            sAssoc = new String(sAssoc + stCurves.stItem[i].sAssoc[j]);
          }
        }

        iTool = -1;
        for (int j=0; j<iStandard; j++)
        {
          switch (iData)
          {
            case _LAS_TYPE:
              sKIDo = las.lasStandardTools.LAS_TOOLS[j][0];
              break;
            case _CORE_TYPE:
              sKIDo = rock.rockStandardTools.ROCK_TOOLS[j][6];
              break;
            case _TOPS_TYPE:
              sKIDo = horizon.strat.stratStandardTools.CURVES[j][0];
              break;
            case _CNTRL_TYPE:
              sKIDo = iqstrat.iqstratControlStandardTools.CURVES[j][0];
              break;
            case _GEO_TYPE:
              sKIDo = iqstrat.iqstratRemarkStandardTools.CURVES[j][0];
              break;
          }

          if (sKID.equals(sKIDo))
            iTool = j;
        }
      }

      if (iTool > -1) iSelected[iTool] = i;
    }
  }

  /** Method removeQuotes()
   * <p> This method will remove the Quotes around the String
   * @param  str  = The string to be converted
   * @param  ch   = character to be removed from string
   * @return sNew = The text converted
   */

  public static String removeChar( String str, char ch )
  {
    String sNew = new String("");
    char   c[]  = null;

    if (str != null)
    {
      c = str.toCharArray();

      for (int i=0; i<c.length; i++)
      {
        if (c[i] != ch)
          sNew = new String(sNew + c[i]);
      }
    }

    return (sNew);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stStruct = null; // Global IQSTRAT Data Structure
    stCMN    = null; // Common Data Structure
    stLAS3   = null; // LAS 3 File Data Structure
    stLAS    = null; // LAS File Data Structure

    // Global Variables

    iArray = 0;                       // Array Number
    iType  = las3.las3Constants._NONE; // Type of Data Identifier
    sType  = null;                    // Type of Data Name Set
    sLabel = null;

    iSelected = null;

    if (stCurves != null)
      stCurves.delete();
    stCurves = null;

    iTotal    = 0;
  }
}

/*
 *  @version 1.1 03/05/2013
 *  @author  John Victorine
 */
