/*
 * @lithologyPlotTrack.java Version 1.1 09/05/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package lith.lithology.plot;

import java.awt.*;
import java.awt.geom.*;

import iqstrat.iqstratHeadersStruct;

import las.lasFileDataStruct;

import lith.lithology.lithologySymbolsListStruct;
import lith.lithology.lithologyListStruct;
import lith.lithology.lithologyStruct;
import lith.lithology.plot.lithologyPlotSymbol;

import xsection.xsectionDataListStruct;

/** CLASS lithologyPlotTrack
 *  <p> This Class will create the Colorlith Plots.
 *
 *  @version 1.1 09/05/2007
 *  @author  John R. Victorine
 */

public class lithologyPlotTrack extends Canvas
{
  // Input Variables

  private double depthStart = 0.0;   // Starting Depth
  private double depthEnd   = 0.0;   // Ending Depth
  private int    iScale     = -1;    // Plot Scale

  // Global Variables

  // Lithology Log Curve Track Data

  private xsectionDataListStruct stXList       = null; // Cross Section Data

  // By Datum

  private int    iDatum    = iqstrat.iqstratTracksStruct._BY_LOG; // Datum
  private String sDatum    = "";

  // Plot Symbol classes

  private lithologySymbolsListStruct stSymbols = null;
  private lithologyPlotSymbol plot             = null;

  // Plot Width Variables

  private int iWidth      = 100;  // Width of Plot
  private int iHeight     = 100;  // Height of Plot with the Titles
  private int iLogHeight  = 100;  // Height of Plot without the Titles
  private int iIncrementY = 100;  // Increment Height

  public static final int PLOT_TITLES = iqstrat.iqstratTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = iqstrat.iqstratTracksStruct.LABELSTART;

  /** CONSTRUCTOR lithologyPlotTrack( double depthStart,
   *                                  double depthEnd,
   *                                  int iScale,
   *                                  lithologySymbolsListStruct stSymbols )
   *  <p> This is the constructor for this class.
   *  @param depthStart  = The starting depth of plot
   *  @param depthEnd    = The ending depth of plot
   *  @param iScale      = The scale
   *  @param stSymbols   = List of Plot Symbols for Lithology
   */

  public lithologyPlotTrack( double depthStart,
                             double depthEnd,
                             int iScale,
                             lithologySymbolsListStruct stSymbols )
  {
    this.stSymbols = stSymbols;

    plot = new lithologyPlotSymbol( stSymbols );

    setPlotHeight(iScale, depthEnd, depthStart);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
    stSymbols = null;
    stXList   = null;

    if (plot != null)
      plot.close();
    plot = null;
  }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setPlotHeight(int iScale, double dMaximum, double dMinimum)
   *  <p> This method will set the Plot Height.
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setPlotHeight(int iScale, double dMaximum, double dMinimum)
  {
    this.depthStart = dMinimum;
    this.depthEnd   = dMaximum;
    this.iScale     = iScale;

    iLogHeight = (int) ((100 * Math.abs( dMaximum - dMinimum )) /
                        iqstrat.iqstratTracksStruct.SCALE[iScale]);
    iHeight    = PLOT_TITLES + iLogHeight;
  }

  /** Method setData()
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    this.stXList = stXList;
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    this.iDatum = iDatum;
    this.sDatum = new String( sDatum );
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawGrid( Graphics g, int iStartTrack,  int iWidthTrack )
   * <p> This method will draw the depth scale
   * @param g           = Graphics Parameter.
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawGrid( Graphics g, int iStartTrack,  int iWidthTrack, int iXData )
  {
    int    i=0;
    int    j=0;
    int    jXinc  = 0;
    int    iIncrement = iWidthTrack/10;
    int    iEndTrack  = iStartTrack+iWidthTrack;
    int    iY1        = 0;
    int    iY2        = 0;
    int    iPercent   = 10;

    // Cross Section Data Variables

    int iDataType  = iqstrat.iqstratHeadersStruct.WELL;
    int iColorlith = iqstrat.iqstratTracksStruct._COLORLITH_RU;
    iqstratHeadersStruct stHeader = null;
    lasFileDataStruct    stLAS    = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        stLAS = stXList.stItem[iXData].stLAS;
        if (stLAS != null)
        {
          if (stLAS.iPE == -1)
            iColorlith = iqstrat.iqstratTracksStruct._COLORLITH_PD;
          else
            iColorlith = iqstrat.iqstratTracksStruct._COLORLITH_RU;
        }

        stHeader = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 10);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    for (i=0; i<=iLogHeight; i+=iIncrementY)
    {
      if (iDataType == iqstrat.iqstratStruct._OUTCROP)
        iY1 = iHeight-i;
      else
        iY1 = i+PLOT_TITLES;

      jXinc = iIncrementY / 5;
      for (j=0; j<5; j++)
      {
        if (iDataType == iqstrat.iqstratStruct._OUTCROP)
          iY2 = iHeight-(i+(j*jXinc));
        else
          iY2 = i+(j*jXinc)+PLOT_TITLES;

        if ((iY2 >= PLOT_TITLES)  && (iY2 < iHeight))
        {
          g.setColor(Color.lightGray);
          g.drawLine(iStartTrack, iY2, iEndTrack, iY2);
        }
      }

      g.setColor(Color.gray);
      g.drawLine(iStartTrack, iY1, iEndTrack, iY1);
    }

    g.setColor(Color.lightGray);
    g.drawLine(iEndTrack, LABELSTART, iEndTrack, PLOT_TITLES+iLogHeight); //iHeight);

    for (i=0; i<=iWidthTrack; i+=iIncrement)
    {
      if ((i==0) || (i==iWidthTrack))
      {
        g.drawLine(i+iStartTrack, LABELSTART,
                   i+iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
      }
      else
      {
        g.drawLine(i+iStartTrack, PLOT_TITLES,
                   i+iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
      }
    }

    g.setColor(Color.black);

    switch (iColorlith)
    {
      case iqstrat.iqstratTracksStruct._COLORLITH_RU:
        g.drawString("Rhomaa-Umaa",iStartTrack+2*iIncrement, LABELSTART+14);
        g.drawString("Computed", iStartTrack+3*iIncrement, LABELSTART+28);
        g.drawString("Rock Column", iStartTrack+2*iIncrement, LABELSTART+42);
        break;
      case iqstrat.iqstratTracksStruct._COLORLITH_RN:
      case iqstrat.iqstratTracksStruct._COLORLITH_PD:
        g.drawString("Rhomaa-NPHI",iStartTrack+2*iIncrement, LABELSTART+14);
        g.drawString("Computed", iStartTrack+3*iIncrement, LABELSTART+28);
        g.drawString("Rock Column", iStartTrack+2*iIncrement, LABELSTART+42);
        break;
    }

    for (i=0; i<=4; i++)
    {
      iPercent = 10 + i*20;
      g.drawString("" + iPercent, iStartTrack + 2*i*iIncrement, PLOT_TITLES);
    }
    g.drawString("Percent", iStartTrack + (7*iIncrement)/2, PLOT_TITLES-14);

    g.setColor(Color.black);
    g.drawLine(iStartTrack, LABELSTART,  iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, PLOT_TITLES, iEndTrack,   PLOT_TITLES);
    g.drawLine(iStartTrack, LABELSTART,  iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEndTrack,   LABELSTART,  iEndTrack,   PLOT_TITLES+iLogHeight); //iHeight);
  }

  /** Method drawLithology( Graphics g, int iStartTrack,  int iWidthTrack )
   *  <p> This method will draw the rock mineral composition percentage track
   * @param g           = Graphics Parameter.
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawLithology( Graphics g, int iStartTrack, int iWidthTrack, int iXData )
  {
    int    i       = 0;
    int    j       = 0;
    int    k       = 0;
    int    n       = 0;
    int    iPlus   = 1;
    int    iDepth1 = 0;
    int    iDepth2 = 0;
    int    iDepth  = 0;
    int    iCount  = 0;
    int    iGroup  = 0;
    double diff    = 0;
    double depth1  = 0.0;
    double depth2  = 0.0;
    double dRatio  = 0.0;

    // Cross Section Data Variables

    int                  iDataType = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev     = 0.0;
    iqstratHeadersStruct stHeader  = null;
    lithologyListStruct  st        = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        st = stXList.stItem[iXData].stLASLithology;

        dElev    = xsection.xsectionDataUtility.getElevation(
                     iDatum, sDatum, stXList.stItem[iXData] );
        stHeader = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    if (st != null)
    {
      if (st.iCount > 0)
      {
        if (st.stItem[0] != null)
        {
          depth1 = st.stItem[0].depthStart;
          depth2 = st.stItem[0].depthEnd;
          diff = depth2 - depth1;
        }

        iPlus = (int) (( 5.0/diff ) *
                       ( iqstrat.iqstratTracksStruct.SCALE[iScale] / 100.0 ));
        if (iPlus < 1) iPlus = 1;

        iCount = -1;
        for (k=0; k<st.iCount; k++)
        {
          if (st.stItem[k] != null)
          {
			if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
			{
              depth1 = st.stItem[k].depthStart;
			}
			else
			{
              depth1 = dElev - st.stItem[k].depthStart;
              if (iDataType == iqstrat.iqstratHeadersStruct.OUTCROP)
                depth1 = dElev + st.stItem[k].depthStart;
			}

            depth2 = depth1 + diff*iPlus;

            for (i=1; i<iPlus; i++)
            {
              if (k < st.iCount-1)
              {
                k++;
              }
            }

            dRatio  = (double) iLogHeight * (depthStart - depth1) /
                                            (depthStart - depthEnd);
            iDepth1 = PLOT_TITLES + (int) dRatio;
            if (iDataType == iqstrat.iqstratStruct._OUTCROP)
              iDepth1 = iHeight - (int) dRatio;

            dRatio  = (double) iLogHeight * (depthStart - depth2) /
                                            (depthStart - depthEnd);
            iDepth2 = PLOT_TITLES + (int) dRatio;
            if (iDataType == iqstrat.iqstratStruct._OUTCROP)
              iDepth2 = iHeight - (int) dRatio;

            iDepth = Math.abs( iDepth2 - iDepth1 );

            if ((iDepth1 >= PLOT_TITLES) && (iDepth2 > PLOT_TITLES) &&
                (iDepth1 < iHeight)      && (iDepth2 < iHeight))
            {
              if (iDataType == iqstrat.iqstratStruct._OUTCROP)
                iDepth1 =  iDepth2;

              iGroup = iDepth / 5;
              if (iGroup == 0) iGroup = 1;

              for (n=0; n<iGroup; n++)
              {
                iCount++;
                for (j=0; j<lith.lithology.lithologyStruct.TOTAL; j++)
                {
                  if (st.stItem[k] != null)
                  {
                    if (st.stItem[k].iLithology != null)
                    {
                      plot.drawCell(g, st.stItem[k].iLithology[j], iCount,
                                    iStartTrack + j * 10, iDepth1 + n * 5);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  /** Method draw( Graphics g, int iSelected, int iStartTrack, int iWidthTrack )
   * <p> This method will create a LAS Plot Track
   * @param g           = Graphics Parameter.
   * @param iSelected   = The Track that was selected
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void draw(Graphics g, int iSelected, int iStartTrack, int iWidthTrack, int iXData )
  {
    int iColor = -1;

    switch (iSelected)
    {
      case iqstrat.iqstratTracksStruct._ROCK_COLUMN:
        iColor = iqstrat.iqstratTracksStruct._SRC_LAS;
        g.setColor( new Color( iqstrat.iqstratTracksStruct.COLORS[iColor][0],
                               iqstrat.iqstratTracksStruct.COLORS[iColor][1],
                               iqstrat.iqstratTracksStruct.COLORS[iColor][2]));
        g.fillRect( iStartTrack, LABELSTART, iWidthTrack, 20 );

        drawLithology( g, iStartTrack, iWidthTrack, iXData );
        drawGrid( g, iStartTrack, iWidthTrack, iXData );
        break;

      case iqstrat.iqstratTracksStruct._ROCK_COLUMN_PD:
        iColor = iqstrat.iqstratTracksStruct._SRC_LAS;
        g.setColor( new Color( iqstrat.iqstratTracksStruct.COLORS[iColor][0],
                               iqstrat.iqstratTracksStruct.COLORS[iColor][1],
                               iqstrat.iqstratTracksStruct.COLORS[iColor][2]));
        g.fillRect( iStartTrack, LABELSTART, iWidthTrack, 20 );

        drawLithology( g, iStartTrack, iWidthTrack, iXData );
        drawGrid( g, iStartTrack, iWidthTrack, iXData );
        break;
    }
  }

  /** Method paint( Graphics g )
   * <p> This method will paint Porosity & Permeability Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
  }
}