/*
 * @lithMathRules.java Version 1.1 07/30/2007
 *
 * Copyright (c) 2006 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package lith.math;

/** Class lithMathRules
 *  <p> This Class will test the log curves and decide what the rock type is.
 *
 *  @version 1.1 07/30/2007
 *  @author  John R. Victorine
 */

public class lithMathRules
{
  public static final double PHI_N4 = -0.04; // -4 %
  public static final double PHI_N2 = -0.02; // -2 %
  public static final double PHI_0  = 0.0;   //  0 %
  public static final double PHI_2  = 0.02;  //  2 %
  public static final double PHI_4  = 0.04;  //  4 %
  public static final double PHI_5  = 0.05;  //  5 %
  public static final double PHI_6  = 0.06;  //  6 %
  public static final double PHI_7  = 0.07;  //  7 %
  public static final double PHI_8  = 0.08;  //  8 %
  public static final double PHI_9  = 0.09;  //  9 %
  public static final double PHI_10 = 0.10;  // 10 %
  public static final double PHI_11 = 0.11;  // 10 %
  public static final double PHI_15 = 0.15;  // 15 %
  public static final double PHI_20 = 0.20;  // 20 %
  public static final double PHI_25 = 0.25;  // 25 %
  public static final double PHI_30 = 0.30;  // 30 %
  public static final double PHI_35 = 0.35;  // 35 %
  public static final double PHI_45 = 0.45;  // 45 %

  public static final double RHO_COAL      = 1.8;
  public static final double RHO_SALT      = 2.03;
  public static final double RHO_GYPSUM    = 2.34;
  public static final double RHO_CLAY      = 2.40;
  public static final double RHO_DOLOMITE  = 2.84;
  public static final double RHO_ANHYDRITE = 2.9;
  public static final double RHO_IGNEOUS   = 2.91;

  public static final double PE_ANHYDRITE  = 4.5;
  public static final double PE_IRONSTONE  = 9.0;
  public static final double PE_IGNEOUS    = 6.0; //4.7;

  /* ======================================================================= *
   * ------------------------------- GET DATA ------------------------------ *
   * ======================================================================= */

  /** Method getLithology()
   * <p> This method will get the Lithology of the rock
   * @param  iH2O       = Indicator for fresh or saline water
   * @param  dShaly     = Start of Shaly gamma ray value
   * @param  dShale     = Start of Shale gamma ray value
   * @param  dHotShale  = Start of Hot Shale gamma ray value
   * @param  dGR        = Gamma Ray Log Value
   * @param  dNPHI      = Neutron Porosity Log Value
   * @param  dRhob      = Bulk Density Log Value
   * @param  dPE        = Photoelectric Log Value
   * @param  dTh        = Spectral Gamma Ray - Thorium
   * @param  dU         = Spectral Gamma Ray - Uranium
   * @param  dK         = Spectral Gamma Ray - Potassium
   * @param  dNull      = Log Curve Null Value
   * @return iLithology = the lithology of the rock
   */

  public static int getLithology(int    iH2O,
                                 double dShaly,
                                 double dShale,
                                 double dHotShale,
                                 double dGR,
                                 double dNPHI,
                                 double dRhob,
                                 double dPE,
                                 double dTh,
                                 double dU,
                                 double dK,
                                 double dNull)
  {
    int    iLithology = lith.lithology.lithologySymbolsStruct._NONE;
    double dDPHI      = PHI_0;
    double delta      = PHI_0;

    dDPHI  = math.mathLAS.computePHI( dRhob,
                                      math.mathLAS._DENSITY,
                                      math.mathLAS._CALCITE,
                                      math.mathLAS._FRESH );
//    dDPHI = (lith.math.lithMathLithology.RHO_MATRIX - dRhob) /
//            (lith.math.lithMathLithology.RHO_MATRIX -
//                 lith.math.lithMathLithology.RHO[iH2O]);
    delta = dNPHI - dDPHI;

    if (dGR >= dHotShale)
    {
      iLithology = lith.lithology.lithologySymbolsStruct._SHALE;

//      if ((delta < PHI_N4) && (Math.abs(dNPHI) < PHI_10))//PHI_2))
      if ((delta < PHI_0) && (Math.abs(dNPHI) < PHI_2))
        iLithology = lith.lithology.lithologySymbolsStruct._ARKOSE;
    }
    else if ((dGR >= dShale) && (dGR < dHotShale))
    {
      iLithology = lith.lithology.lithologySymbolsStruct._SHALE;
//      if ((Math.abs(dNPHI) >= PHI_0) && (Math.abs(dNPHI) < PHI_4))//PHI_2))

      if ((delta >= PHI_N4) && (delta <=PHI_4) &&
          (Math.abs(dNPHI) >= PHI_10) && (Math.abs(dNPHI) <= PHI_30))
        iLithology = lith.lithology.lithologySymbolsStruct._SHALY_SAND;

//      if ((delta < PHI_N2) && (Math.abs(dNPHI) < PHI_10))
      if ((delta < PHI_0) && (Math.abs(dNPHI) < PHI_2))
        iLithology = lith.lithology.lithologySymbolsStruct._ARKOSE;
    }
    else
    {
      if (delta < PHI_N4)
      {
        iLithology = lith.lithology.lithologySymbolsStruct._SANDSTONE;
        if ( (dGR >= dShaly) && (dGR < dShale))
          iLithology = lith.lithology.lithologySymbolsStruct._SHALY_SAND;

        if ((dRhob <= RHO_SALT) && (Math.abs(dNPHI) <= PHI_5))
          iLithology = lith.lithology.lithologySymbolsStruct._HALITE;

        if ((dRhob < RHO_COAL) && (dNPHI > PHI_35))
          iLithology = lith.lithology.lithologySymbolsStruct._COAL;
      }

      else if ((delta >= PHI_N4) && (delta <=PHI_4))
      {
        iLithology = lith.lithology.lithologySymbolsStruct._LIMESTONE;
        if ( (dGR >= dShaly) && (dGR < dShale))
          iLithology = lith.lithology.lithologySymbolsStruct._SHALY_CARB;
      }

      else if (delta > PHI_4)
      {
        iLithology = lith.lithology.lithologySymbolsStruct._DOLOMITE;
        if ((dGR >= dShaly) && (dGR < dShale))
          iLithology = lith.lithology.lithologySymbolsStruct._SHALY_CARB;

        if ((dRhob >= RHO_GYPSUM) && (dNPHI > PHI_45))
          iLithology = lith.lithology.lithologySymbolsStruct._GYPSUM;

        if ((dRhob > RHO_CLAY) && (dNPHI > PHI_45))
          iLithology = lith.lithology.lithologySymbolsStruct._SCHIST;
//          iLithology = lith.lithology.lithologySymbolsStruct._CLAY;

        if (dRhob > RHO_DOLOMITE)
        {
          iLithology = lith.lithology.lithologySymbolsStruct._DOLOMITE;
          if ((dGR >= dShaly) && (dGR < dShale))
            iLithology = lith.lithology.lithologySymbolsStruct._SHALY_CARB;
        }

        if (((dRhob > RHO_ANHYDRITE) || (dPE > PE_ANHYDRITE)) &&
            (Math.abs(dNPHI) <= PHI_5))
          iLithology = lith.lithology.lithologySymbolsStruct._ANHYDRITE;

        if (((dRhob > RHO_ANHYDRITE) || (dPE > PE_IRONSTONE)) &&
            (dNPHI > PHI_15))
          iLithology = lith.lithology.lithologySymbolsStruct._IRONSTONE;

        if (((dRhob > RHO_IGNEOUS) || (dPE > PE_IGNEOUS)) &&
            (dNPHI > PHI_10))
          iLithology = lith.lithology.lithologySymbolsStruct._IGNEOUS;
      }
    }

    return (iLithology);
  }
}