/*
 * @lithPlotColorlith.java Version 1.1 09/05/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package lith.plot;

import java.awt.*;
import java.awt.geom.*;

import iqstrat.iqstratHeadersStruct;
import las.lasFileDataStruct;
import xsection.xsectionDataListStruct;

/** CLASS lithPlotColorlith
 *  <p> This Class will create the Colorlith Plots.
 *
 *  @version 1.1 09/05/2007
 *  @author  John R. Victorine
 */

public class lithPlotColorlith extends Canvas
{
  // Input Variables

  private double depthStart = 0.0;   // Starting Depth
  private double depthEnd   = 0.0;   // Ending Depth
  private int    iScale     = -1;    // Plot Scale

  // Global Variables

  // Data Variable

  private xsectionDataListStruct stXList = null;

  // By Datum

  private int    iDatum    = iqstrat.iqstratTracksStruct._BY_LOG; // Datum
  private String sDatum    = "";

  public static final double RHO_MAX = 3.0;
  public static final double RHO_MIN = 2.0;

  // Rhomaa-Umaa Plot Limits

  public static final double UMAA_MAX   = iqstrat.iqstratTracksStruct.UMAA_MAX;
  public static final double UMAA_MIN   = iqstrat.iqstratTracksStruct.UMAA_MIN;

  public static final double RHOMAA_MAX = iqstrat.iqstratTracksStruct.RHOMAA_MAX;
  public static final double RHOMAA_MIN = iqstrat.iqstratTracksStruct.RHOMAA_MIN;

  // Porosity Difference Plot Limits

  public static final double PHI_MAX  = iqstrat.iqstratTracksStruct.PHI_MAX;
  public static final double PHI_MIN  = iqstrat.iqstratTracksStruct.PHI_MIN;

  public static final double DPHI_MAX = iqstrat.iqstratTracksStruct.DPHI_MAX;
  public static final double DPHI_MIN = iqstrat.iqstratTracksStruct.DPHI_MIN;

  // Spectral Gamma Ray Ratio Plot Limits

  public static final double THU_MAX  = iqstrat.iqstratTracksStruct.THU_MAX;
  public static final double THU_MIN  = iqstrat.iqstratTracksStruct.THU_MIN;

  public static final double THK_MAX  = iqstrat.iqstratTracksStruct.THK_MAX;
  public static final double THK_MIN  = iqstrat.iqstratTracksStruct.THK_MIN;

  // Spectral Gamma Ray Plot Limits

  public static final double U_MAX    = iqstrat.iqstratTracksStruct.U_MAX;
  public static final double U_MIN    = iqstrat.iqstratTracksStruct.U_MIN;

  public static final double TH_MAX   = iqstrat.iqstratTracksStruct.TH_MAX;
  public static final double TH_MIN   = iqstrat.iqstratTracksStruct.TH_MIN;

  public static final double K_MAX    = iqstrat.iqstratTracksStruct.K_MAX;
  public static final double K_MIN    = iqstrat.iqstratTracksStruct.K_MIN;

  // Plot Variables

  private int iWidth      = 100;  // Width of Plot
  private int iHeight     = 100;  // Height of Plot with the Titles
  private int iLogHeight  = 100;  // Height of Plot without the Titles
  private int iIncrementY = 100;  // Increment Height

  public static final int PLOT_TITLES = iqstrat.iqstratTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = iqstrat.iqstratTracksStruct.LABELSTART;

  /** CONSTRUCTOR lithPlotColorlith( double depthStart,
   *                                 double depthEnd,
   *                                 int iScale )
   *  <p> This is the constructor for this class.
   *  @param depthStart  = The starting depth of plot
   *  @param depthEnd    = The ending depth of plot
   *  @param iScale      = The scale
   */

  public lithPlotColorlith( double depthStart,
                            double depthEnd,
                            int iScale )
  {
    setPlotHeight(iScale, depthEnd, depthStart);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close() { stXList = null; }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setPlotHeight( int iScale, double dMaximum, double dMinimum )
   *  <p> This method will set the Plot Height.
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setPlotHeight( int iScale, double dMaximum, double dMinimum )
  {
    this.depthStart = dMinimum;
    this.depthEnd   = dMaximum;
    this.iScale     = iScale;

    iLogHeight = (int) ((100 * Math.abs( dMaximum - dMinimum )) /
                        iqstrat.iqstratTracksStruct.SCALE[iScale]);
    iHeight    = PLOT_TITLES + iLogHeight;
  }

  /** Method setData()
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    this.stXList = stXList;
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    this.iDatum = iDatum;
    this.sDatum = new String( sDatum );
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawGammaTrack()
   *  <p> This method will color the zones in the Lithology Track as gray levels.
   * @param g           = Graphics Parameter.
   * @param iSelected   = The Log Lithology Track Selected
   * @param iStartTrack = The starting pixel for the Lithology Track
   * @param iWidthTrack = The width of Lithology Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawGammaTrack( Graphics g,
                              int iSelected,
                              int iStartTrack,
                              int iWidthTrack,
                              int iXData )
  {
    int    iDepth1  = 0;
    int    iDepth2  = 0;
    int    iDepth   = 0;
    int    iGColor  = 0;
    int    iForm    = 0;
    double dRatio   = 0.0;
    double dLog     = 0.0;
    double depth1   = 0.0;
    double depth2   = 0.0;
    int    iEndTrack = iStartTrack+iWidthTrack;
    double dGammaMin = las.lasFileDataStruct.GAMMA_MIN;
    double dGammaMax = las.lasFileDataStruct.GAMMA_MAX;
    int    iRows     = 0;
    double depths[]  = null;
    double dCurves[] = null;

    int    iRed      = 255;
    int    iGreen    = 255;
    int    iBlue     = 255;
    double dShaly    = las.lasFileDataStruct.SHALY;
    double dShale    = las.lasFileDataStruct.SHALE;
    double diff      = 0.0;

    // Cross Section Data Variables

    int                  iDataType      = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev          = 0.0;
    iqstratHeadersStruct stHeader       = null;
    lasFileDataStruct    stLASFileData  = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        dElev          = xsection.xsectionDataUtility.getElevation(
                           iDatum, sDatum, stXList.stItem[iXData] );
        stLASFileData  = stXList.stItem[iXData].stLAS;
        stHeader       = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    g.setColor(new Color(255, 255, 255));

    if (iSelected == iqstrat.iqstratTracksStruct._LITHOLOGY_GR)
    {
      if (stLASFileData != null)
      {
        iRows   = stLASFileData.iRows;
        depths  = stLASFileData.depths;
        dCurves = stLASFileData.dGR;
      }
    }

    if (iRows > 0)
    {
      for (int i=0; i<iRows-1; i++)
      {
    	if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
		{
          depth1 = depths[i];
          depth2 = depths[i+1];
	    }
	    else
	    {
          depth1 = dElev - depths[i];
          depth2 = dElev - depths[i+1];

          if (iDataType == iqstrat.iqstratHeadersStruct.OUTCROP)
          {
            depth1 = dElev + depths[i];
            depth2 = dElev + depths[i+1];
          }
		}

        dRatio  = (double) iLogHeight * (depthStart - depth1) /
                                        (depthStart - depthEnd);
        iDepth1 = PLOT_TITLES + (int) dRatio;
        if (iDataType == iqstrat.iqstratStruct._OUTCROP)
          iDepth1 = iHeight - (int) dRatio;

        dRatio  = (double) iLogHeight * (depthStart - depth2) /
                                        (depthStart - depthEnd);
        iDepth2 = PLOT_TITLES + (int) dRatio;
        if (iDataType == iqstrat.iqstratStruct._OUTCROP)
          iDepth2 = iHeight - (int) dRatio;

        iDepth = Math.abs( iDepth2 - iDepth1 );

        dLog = dCurves[i];

        if (dLog < dGammaMin) dLog = dGammaMin;
        if (dLog > dGammaMax) dLog = dGammaMax;

        if (dLog < dShale)
        {
		  diff   = dShale - dGammaMin;
		  iRed   = 255;
		  iGreen = (int) ( 255 * (dShale - dLog) / diff );
		  iBlue  = 0;
		}
		else
		{
		  diff   = dGammaMax - dShale;
		  iRed   = (int) ( 255 * (dGammaMax - dLog) / diff );
		  iGreen = 0;
		  iBlue  = 0;
		}

        g.setColor(new Color(iRed, iGreen, iBlue));

        iGColor = (int) (255 * (1.0 - ((dLog-dGammaMin) / (dGammaMax-dGammaMin))));

        if ((iDepth1 > PLOT_TITLES) && (iDepth2 > PLOT_TITLES) &&
            (iDepth1 < iHeight)     && (iDepth2 < iHeight))
        {
          iForm = (iWidthTrack * (256-iGColor)) / 255;
          iForm = iWidthTrack - iForm / 2;
          g.fillRect(iEndTrack-iForm, iDepth1, iForm, iDepth);
        }
      }
    }

    // Draw Gamma Ray Grid & Title

    g.setColor(Color.black);

    Font        fsb  = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );

    Graphics2D g2 = (Graphics2D) g;
    g2.rotate( (Math.PI / 2.0));
    g2.drawString("Lithology", LABELSTART+14, -1*(iStartTrack+2*iWidthTrack/3));
    g2.drawString("Gamma Ray", LABELSTART+7, -1*(iStartTrack+5));
    g2.rotate( -1.0 * (Math.PI / 2.0));

    g.drawLine(iStartTrack, LABELSTART,  iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, PLOT_TITLES, iEndTrack,   PLOT_TITLES);
    g.drawLine(iStartTrack, LABELSTART,  iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEndTrack,   LABELSTART,  iEndTrack,   PLOT_TITLES+iLogHeight); //iHeight);
  }

  /** Method drawLithologyTrack()
   *  <p> This method will color the zones in the Lithology Track.
   * @param g           = Graphics Parameter.
   * @param iSelected   = The Track that was selected
   * @param iStartTrack = The starting pixel for the Lithology Track
   * @param iWidthTrack = The width of Lithology Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawLithologyTrack( Graphics g,
                                  int iSelected,
                                  int iStartTrack,
                                  int iWidthTrack,
                                  int iXData )
  {
    int    iRows     = 0;
    int    iDepth1   = 0;
    int    iDepth2   = 0;
    int    iDepth    = 0;
    int    iRed      = 0;
    int    iGreen    = 0;
    int    iBlue     = 0;
    int    iTemp     = 0;
    int    iForm     = 0;
    double depth1    = 0.0;
    double depth2    = 0.0;
    double dLog      = 0.0;
    double dRatio    = 0.0;
    int    iEndTrack = iStartTrack+iWidthTrack;

    double dRedMax   = 0.0;
    double dRedMin   = 0.0;
    double dGreenMax = 0.0;
    double dGreenMin = 0.0;
    double dBlueMax  = 0.0;
    double dBlueMin  = 0.0;

    double dNull     = 0.0;
    double depths[]  = null;
    double dRed[]    = null;
    double dGreen[]  = null;
    double dBlue[]   = null;

    String sTitle1   = "";
    String sTitle2   = "";

    // Cross Section Data Variables

    int                  iDataType      = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev          = 0.0;
    iqstratHeadersStruct stHeader       = null;
    lasFileDataStruct    stLASFileData  = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        dElev          = xsection.xsectionDataUtility.getElevation(
                           iDatum, sDatum, stXList.stItem[iXData] );
        stLASFileData  = stXList.stItem[iXData].stLAS;
        stHeader       = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    switch (iSelected)
    {
      case iqstrat.iqstratTracksStruct._CLRLITH_RU:
        if (stLASFileData != null)
        {
          iRows     = stLASFileData.iRows;
          dNull     = stLASFileData.dNull;
          depths    = stLASFileData.depths;

          dRed      = stLASFileData.dRhomaa;
          dRedMax   = RHOMAA_MAX;
          dRedMin   = RHOMAA_MIN;

          dGreen    = stLASFileData.dUmaa;
          dGreenMax = UMAA_MAX;
          dGreenMin = UMAA_MIN;

          dBlue     = stLASFileData.dGR;
          dBlueMax  = las.lasFileDataStruct.GAMMA_MAX;
          dBlueMin  = las.lasFileDataStruct.GAMMA_MIN;
        }

        sTitle1   = "Colorlith";
        sTitle2   = "Rhomaa-Umaa";
        break;

      case iqstrat.iqstratTracksStruct._CLRLITH_RN:
        if (stLASFileData != null)
        {
          iRows     = stLASFileData.iRows;
          dNull     = stLASFileData.dNull;
          depths    = stLASFileData.depths;

          dRed      = stLASFileData.dRhomaa;
          dRedMax   = RHOMAA_MAX;
          dRedMin   = RHOMAA_MIN;

          dGreen    = stLASFileData.dNPHI;
          dGreenMax = PHI_MAX;
          dGreenMin = PHI_MIN;

          dBlue     = stLASFileData.dGR;
          dBlueMax  = las.lasFileDataStruct.GAMMA_MAX;
          dBlueMin  = las.lasFileDataStruct.GAMMA_MIN;
        }

        sTitle1   = "Colorlith";
        sTitle2   = "Rhomaa-NPHI";
        break;

      case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
        if (stLASFileData != null)
        {
          iRows     = stLASFileData.iRows;
          dNull     = stLASFileData.dNull;
          depths    = stLASFileData.depths;

          dRed      = stLASFileData.dThU;
          dRedMax   = Math.log(THU_MAX)/Math.log(10.0);
          dRedMin   = Math.log(THU_MIN)/Math.log(10.0);

          dGreen    = stLASFileData.dThK;
          dGreenMax = Math.log(THK_MAX)/Math.log(10.0);
          dGreenMin = Math.log(THK_MIN)/Math.log(10.0);

          dBlue     = stLASFileData.dGR;
          dBlueMax  = las.lasFileDataStruct.GAMMA_MAX;
          dBlueMin  = las.lasFileDataStruct.GAMMA_MIN;
        }

        sTitle1   = "Colorlith";
        sTitle2   = "Th/U-Th/K";
        break;

      case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
        if (stLASFileData != null)
        {
          iRows     = stLASFileData.iRows;
          dNull     = stLASFileData.dNull;
          depths    = stLASFileData.depths;

          dRed      = stLASFileData.dK;
          dRedMax   = K_MAX;
          dRedMin   = K_MIN;

          dGreen    = stLASFileData.dTh;
          dGreenMax = TH_MAX;
          dGreenMin = TH_MIN;

          dBlue     = stLASFileData.dU;
          dBlueMax  = U_MAX;
          dBlueMin  = U_MIN;
        }

        sTitle1   = "Colorlith";
        sTitle2   = "K-Th-U Hall";
        break;
    }

    // Start Lithology Track

    g.setColor(new Color(255, 255, 255));

    if (iRows > 0)
    {
      for (int i=0; i<iRows-1; i++)
      {
        if ((dRed[i] > dNull) && (dGreen[i] > dNull))
        {
      	  if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
		  {
            depth1 = depths[i];
            depth2 = depths[i+1];
	      }
	      else
	      {
            depth1 = dElev - depths[i];
            depth2 = dElev - depths[i+1];

            if (iDataType == iqstrat.iqstratHeadersStruct.OUTCROP)
            {
              depth1 = dElev + depths[i];
              depth2 = dElev + depths[i+1];
            }
		  }

          dRatio  = (double) iLogHeight * (depth1-depthStart) /
                                          (depthEnd - depthStart);
          iDepth1 = PLOT_TITLES + (int) dRatio;
          if (iDataType == iqstrat.iqstratStruct._OUTCROP)
            iDepth1 = iHeight - (int) dRatio;

          dRatio  = (double) iLogHeight * (depth2-depthStart) /
                                          (depthEnd - depthStart);
          iDepth2 = PLOT_TITLES + (int) dRatio;
          if (iDataType == iqstrat.iqstratStruct._OUTCROP)
            iDepth2 = iHeight - (int) dRatio;

          iDepth = Math.abs( iDepth2 - iDepth1 );

          // Set Blue Color Level

          dLog = dBlue[i];

          if (dLog < dBlueMin) dLog = dBlueMin;
          if (dLog > dBlueMax) dLog = dBlueMax;

          iBlue = (int) (255 * (dLog-dBlueMin) / (dBlueMax-dBlueMin));

          // Set Red Color Level

          switch (iSelected)
          {
            case iqstrat.iqstratTracksStruct._CLRLITH_RU:
            case iqstrat.iqstratTracksStruct._CLRLITH_RN:
            case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
              dLog = dRed[i];
              break;

            case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
              dLog = Math.log(dRed[i]) / Math.log(10.0);
              break;
          }

          if (dLog > dRedMax) dLog = dRedMax;
          if (dLog < dRedMin) dLog = dRedMin;

          switch (iSelected)
          {
            case iqstrat.iqstratTracksStruct._CLRLITH_RU:
            case iqstrat.iqstratTracksStruct._CLRLITH_RN:
              iRed = (int) (255.0*(1.0 - ((dLog-dRedMin) / (dRedMax-dRedMin))));
              break;

            case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
            case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
              iRed = (int) (255.0 * ((dLog-dRedMin) / (dRedMax-dRedMin)));
              break;
          }

          if (iRed > 255) iRed = 255;
          if (iRed < 0)   iRed = 0;

          // Set Green Color Level

          switch (iSelected)
          {
            case iqstrat.iqstratTracksStruct._CLRLITH_RU:
            case iqstrat.iqstratTracksStruct._CLRLITH_RN:
            case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
              dLog = dGreen[i];
              break;

            case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
              dLog = Math.log(dGreen[i]) / Math.log(10.0);
              break;
          }

          if (dLog > dGreenMax) dLog = dGreenMax;
          if (dLog < dGreenMin) dLog = dGreenMin;

          switch (iSelected)
          {
            case iqstrat.iqstratTracksStruct._CLRLITH_RU:
            case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
            case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
              iGreen = (int)(255.0 * (dLog-dGreenMin) / (dGreenMax-dGreenMin) );
              break;
            case iqstrat.iqstratTracksStruct._CLRLITH_RN:
              iGreen = (int)(255.0 * (1.0 -
                              ((dLog-dGreenMin) / (dGreenMax-dGreenMin))));
              break;
          }

          if (iGreen > 255) iGreen = 255;
          if (iGreen < 0)   iGreen = 0;

          // Set RGB Color


          switch (iSelected)
          {
            case iqstrat.iqstratTracksStruct._CLRLITH_RU:
            case iqstrat.iqstratTracksStruct._CLRLITH_RN:
            case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
              g.setColor(new Color(iRed, iGreen, iBlue));
              break;
            case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
              g.setColor(new Color(iRed, iGreen, 0));
              break;
          }

          // Draw Colorlith Block

          if ((iDepth1 > PLOT_TITLES) && (iDepth2 > PLOT_TITLES) &&
              (iDepth1 < iHeight)     && (iDepth2 < iHeight))
          {
            if (iSelected == iqstrat.iqstratTracksStruct._CLRLITH_HALL)
            {
              g.fillRect(iStartTrack, iDepth1, iWidthTrack, iDepth);
            }
            else
            {
              iForm = (iWidthTrack * (iBlue)) / 255;
              iForm = iWidthTrack - iForm / 2;
              g.fillRect(iEndTrack - iForm, iDepth1, iForm, iDepth);
            }
          }
        }
      }
    }

    // Draw Gamma Ray Grid & Title

    g.setColor(Color.black);

    Font        fsb  = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );

    Graphics2D g2 = (Graphics2D) g;
    g2.rotate( (Math.PI / 2.0));
    g2.drawString(sTitle1, LABELSTART+14, -1*(iStartTrack+2*iWidthTrack/3));

    fsb  = new Font("Serif", Font.BOLD, 10);
    fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );

    g2.drawString(sTitle2, LABELSTART+6, -1*(iStartTrack+5));
    g2.rotate( -1.0 * (Math.PI / 2.0));

    g.drawLine(iStartTrack, LABELSTART,  iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, PLOT_TITLES, iEndTrack,   PLOT_TITLES);
    g.drawLine(iStartTrack, LABELSTART,  iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEndTrack,   LABELSTART,  iEndTrack,   PLOT_TITLES+iLogHeight); //iHeight);
  }

  /** Method draw()
   * <p> This method will create a LAS Plot Track
   * @param g           = Graphics Parameter.
   * @param iSelected   = The Track that was selected
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void draw( Graphics g,
                    int iSelected,
                    int iStartTrack,
                    int iWidthTrack,
                    int iXData)
  {
    int iColor = iqstrat.iqstratTracksStruct._SRC_LAS;

    if (iSelected == iqstrat.iqstratTracksStruct._LITHOLOGY_GR)
    {
      switch (iSelected)
      {
        case iqstrat.iqstratTracksStruct._LITHOLOGY_GR:
          iColor = iqstrat.iqstratTracksStruct._SRC_LAS;
          break;
      }

      g.setColor( new Color( iqstrat.iqstratTracksStruct.COLORS[iColor][0],
                             iqstrat.iqstratTracksStruct.COLORS[iColor][1],
                             iqstrat.iqstratTracksStruct.COLORS[iColor][2]));
      g.fillRect(iStartTrack, LABELSTART, iWidthTrack, 20);

      drawGammaTrack(g, iSelected, iStartTrack, iWidthTrack, iXData);
    }

    if ((iSelected == iqstrat.iqstratTracksStruct._CLRLITH_RU) ||
        (iSelected == iqstrat.iqstratTracksStruct._CLRLITH_RN) ||
        (iSelected == iqstrat.iqstratTracksStruct._CLRLITH_SPGR) ||
        (iSelected == iqstrat.iqstratTracksStruct._CLRLITH_HALL) )
    {
      switch (iSelected)
      {
        case iqstrat.iqstratTracksStruct._CLRLITH_RU:
        case iqstrat.iqstratTracksStruct._CLRLITH_RN:
        case iqstrat.iqstratTracksStruct._CLRLITH_SPGR:
        case iqstrat.iqstratTracksStruct._CLRLITH_HALL:
          iColor = iqstrat.iqstratTracksStruct._SRC_LAS;
          break;
      }

      g.setColor(new Color(iqstrat.iqstratTracksStruct.COLORS[iColor][0],
                           iqstrat.iqstratTracksStruct.COLORS[iColor][1],
                           iqstrat.iqstratTracksStruct.COLORS[iColor][2]));
      g.fillRect(iStartTrack, LABELSTART, iWidthTrack, 20);

      drawLithologyTrack(g, iSelected, iStartTrack, iWidthTrack, iXData);
    }
  }

  /** Method paint( Graphics g )
   * <p> This method will paint Porosity & Permeability Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
  }
}