/*
 * @rockColorPlotTrack.java Version 1.1 09/10/2007
 *
 * Copyright (c) 2007 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package rock.color.plot;

import java.awt.*;
import java.awt.geom.*;

import iqstrat.iqstratHeadersStruct;

import rock.color.rockColorListStruct;

import xsection.xsectionDataListStruct;

/** CLASS rockColorPlotTrack
 *  <p> This Class will create the Rock Color Track.
 *
 *  @version 1.1 09/10/2007
 *  @author  John R. Victorine
 */

public class rockColorPlotTrack extends Canvas
{
  // Input Variables

  private double depthStart = 0.0;   // Starting Depth
  private double depthEnd   = 0.0;   // Ending Depth
  private int    iScale     = -1;    // Plot Scale

  // Global Variables

  private xsectionDataListStruct stXList = null;

  // By Datum

  private int    iDatum    = iqstrat.iqstratTracksStruct._BY_LOG; // Datum
  private String sDatum    = "";

  // Plot Variables

  private int iWidth      = 100;  // Width of Plot
  private int iHeight     = 100;  // Height of Plot with the Titles
  private int iLogHeight  = 100;  // Height of Plot without the Titles
  private int iIncrementY = 100;  // Increment Height

  public static final int PLOT_TITLES = iqstrat.iqstratTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = iqstrat.iqstratTracksStruct.LABELSTART;

  /** CONSTRUCTOR rockColorPlotTrack( double depthStart,
   *                                  double depthEnd,
   *                                  int iScale )
   *  <p> This is the constructor for this class.
   *  @param depthStart  = The starting depth of plot
   *  @param depthEnd    = The ending depth of plot
   *  @param iScale      = The scale
   */

  public rockColorPlotTrack( double depthStart,
                             double depthEnd,
                             int iScale )
  {
    setPlotHeight(iScale, depthEnd, depthStart);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close() { stXList = null; }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setPlotHeight( int iScale, double dMaximum, double dMinimum )
   *  <p> This method will set the Plot Height.
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setPlotHeight( int iScale, double dMaximum, double dMinimum )
  {
    this.depthStart = dMinimum;
    this.depthEnd   = dMaximum;
    this.iScale     = iScale;

    iLogHeight = (int) ((100 * Math.abs( dMaximum - dMinimum )) /
                        iqstrat.iqstratTracksStruct.SCALE[iScale]);
    iHeight    = PLOT_TITLES + iLogHeight;
  }

  /** Method setData()
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    this.stXList = stXList;
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    this.iDatum = iDatum;
    this.sDatum = new String( sDatum );
  }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawGrid()
   * <p> This method will draw the depth scale
   * @param g           = Graphics Parameter.
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawGrid( Graphics g, int iStartTrack, int iWidthTrack, int iXData )
  {
    int    i=0;
    int    j=0;
    int    jXinc  = 0;
    int    iLog   = 0;
    int    iCycle = 1;
    double dXinc  = 0.0;
    double dTemp  = 0.0;
    int    iIncrement  = iWidthTrack/6;
    int    iEndTrack   = iStartTrack+iWidthTrack;
    int    iY1         = 0;
    int    iY2         = 0;

    // Cross Section Data Variables

    int                  iDataType = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev     = 0.0;
    iqstratHeadersStruct stHeader  = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        dElev    = xsection.xsectionDataUtility.getElevation(
                     iDatum, sDatum, stXList.stItem[iXData] );
        stHeader = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    g.setColor(Color.black);

    for (i=0; i<=iLogHeight; i+=iIncrementY)
    {
      if (iDataType == iqstrat.iqstratStruct._OUTCROP)
        iY1 = iHeight-i;
      else
        iY1 = i+PLOT_TITLES;

      jXinc = iIncrementY / 5;
      for (j=0; j<5; j++)
      {
        if (iDataType == iqstrat.iqstratStruct._OUTCROP)
          iY2 = iHeight-(i+(j*jXinc));
        else
          iY2 = i+(j*jXinc)+PLOT_TITLES;

        if ((iY2 >= PLOT_TITLES)  && (iY2 < iHeight))
        {
          g.setColor(Color.lightGray);
          g.drawLine(iStartTrack, iY2, iEndTrack, iY2);
        }
      }

      g.setColor(Color.gray);
      g.drawLine(iStartTrack, iY1, iEndTrack, iY1);
    }

    g.setColor(Color.black);
    g.drawLine(iStartTrack, LABELSTART, iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, LABELSTART, iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEndTrack,   LABELSTART, iEndTrack,   PLOT_TITLES+iLogHeight); //iHeight);
  }

  /** Method drawRockColorTrack()
   *   <p> This track will plot the rock color track data
   * @param g           = Graphics Parameter.
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void drawRockColorTrack( Graphics g, int iStartTrack, int iWidthTrack, int iXData )
  {
    int    iDepth1     = 0;
    int    iDepth2     = 0;
    int    iDepth      = 0;
    double dRatio      = 0.0;
    double depth1      = 0.0;
    double depth2      = 0.0;
    int    iStart      = iStartTrack;
    int    iEnd        = iStartTrack+iWidthTrack;
    int    iWide       = iWidthTrack;
    int    iStartColor = iStartTrack;
    int    iWidthColor = iWidthTrack;
    int    iIncrement  = iWidthTrack/5;
    Font        fsb       = new Font("Serif", Font.BOLD, 10);
    FontMetrics fsbm      = g.getFontMetrics(fsb);

    g.setFont( fsb );
    g.setColor(Color.black);

    // Cross Section Data Variables

    int                  iDataType = iqstrat.iqstratHeadersStruct.WELL;
    double               dElev     = 0.0;
    iqstratHeadersStruct stHeader  = null;
    rockColorListStruct  stColor   = null;

    if (stXList != null)
    {
      if (stXList.stItem[iXData] != null)
      {
        dElev    = xsection.xsectionDataUtility.getElevation(
                     iDatum, sDatum, stXList.stItem[iXData] );
        stColor  = stXList.stItem[iXData].stColor;
        stHeader = stXList.stItem[iXData].stHeader;
        if (stHeader != null)
        {
          iDataType = stHeader.iType;
        }
      }
    }

    if (stColor != null)
    {
      for (int i=0; i<stColor.iCount; i++)
      {
        if (stColor.stItem[i] != null)
        {
      	  if (iDatum == iqstrat.iqstratTracksStruct._BY_LOG)
		  {
            depth1 = stColor.stItem[i].depthStart;
            depth2 = stColor.stItem[i].depthEnd;
		  }
		  else
		  {
            depth1 = dElev - stColor.stItem[i].depthStart;
            depth2 = dElev - stColor.stItem[i].depthEnd;

            if (iDataType == iqstrat.iqstratHeadersStruct.OUTCROP)
            {
              depth1 = dElev + stColor.stItem[i].depthStart;
              depth2 = dElev + stColor.stItem[i].depthEnd;
            }
		  }

          dRatio = (double) iLogHeight * (depthStart - depth1) /
                                         (depthStart - depthEnd);
          iDepth1 = PLOT_TITLES + (int) dRatio;
          if (iDataType == iqstrat.iqstratStruct._OUTCROP)
            iDepth1 = iHeight - (int) dRatio;

          dRatio = (double) iLogHeight * (depthStart - depth2) /
                                         (depthStart - depthEnd);
          iDepth2 = PLOT_TITLES + (int) dRatio;
          if (iDataType == iqstrat.iqstratStruct._OUTCROP)
            iDepth2 = iHeight - (int) dRatio;

          iDepth = Math.abs(iDepth2 - iDepth1);

          if ((iDepth1 >= PLOT_TITLES) && (iDepth2 > PLOT_TITLES) &&
              (iDepth1 < iHeight)     && (iDepth2 <= iHeight))
          {
//          if (stColor.stItem[i].iTotal == 0)
//            stColor.stItem[i].iTotal = 1;

//          int iDiff = iWide / stColor.stItem[i].iTotal;
//          for (int k=0; k<stColor.stItem[i].iTotal; k++)
            for (int k=0; k<3; k++)
            {
              switch (k)
              {
                case 0:
                  iStartColor = iStart;
                  iWidthColor = iWidthTrack;
                  break;

                case 1:
                  iStartColor = iStart + 3 * iWidthTrack / 5;
                  iWidthColor = 2 * iWidthTrack / 5;
                  break;

                case 2:
                  iStartColor = iStart + 9 * iWidthTrack / 10;
                  iWidthColor = iWidthTrack / 10;
                  break;
              }

              for (int m=0; m<3; m++)
              {
                if (stColor.stItem[i].iRGB[k][m] > 255)
                  stColor.stItem[i].iRGB[k][m] = 255;
              }

              if (stColor.stItem[i].sMunsell[k].length() > 0)
              {
                g.setColor(new Color(stColor.stItem[i].iRGB[k][0],
                                     stColor.stItem[i].iRGB[k][1],
                                     stColor.stItem[i].iRGB[k][2]));
//              g.fillRect(iStart + k * iDiff, iDepth1, iDiff, iDepth);
                g.fillRect(iStartColor, iDepth1, iWidthColor, iDepth);
              }
            }
          }
        }
      }
    }

    g.setColor(Color.black);

    g.drawString("Rock", iStartTrack+iIncrement,  LABELSTART+14);
    g.drawString("Color", iStartTrack+iIncrement, LABELSTART+28);
/*
    Font        fsb  = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);

    g.setFont( fsb );

    Graphics2D g2 = (Graphics2D) g;
    g2.rotate( (Math.PI / 2.0));
    g2.drawString("Rock Color", LABELSTART+14, -1*(iStart+2*iWidthTrack/3));
    g2.rotate( -1.0 * (Math.PI / 2.0));
*/
    g.drawLine(iStart, LABELSTART,  iEnd,   LABELSTART);
    g.drawLine(iStart, PLOT_TITLES, iEnd,   PLOT_TITLES);
    g.drawLine(iStart, LABELSTART,  iStart, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEnd,   LABELSTART,  iEnd,   PLOT_TITLES+iLogHeight); //iHeight);
  }

  /** Method draw()
   * <p> This method will create a LAS Plot Track
   * @param g           = Graphics Parameter.
   * @param iSelected   = The Porosity Track to draw
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   * @param iXData      = The well/outcrop number in Cross Section List Data
   */

  public void draw( Graphics g, int iSelected, int iStartTrack, int iWidthTrack, int iXData )
  {
    int iColor = iqstrat.iqstratTracksStruct._SRC_ROCK;

    if (iSelected == iqstrat.iqstratTracksStruct._ROCK_COLOR)
    {
      g.setColor( new Color( iqstrat.iqstratTracksStruct.COLORS[iColor][0],
                             iqstrat.iqstratTracksStruct.COLORS[iColor][1],
                             iqstrat.iqstratTracksStruct.COLORS[iColor][2] ) );
      g.fillRect( iStartTrack, LABELSTART, iWidthTrack, 20 );

      drawRockColorTrack( g, iStartTrack, iWidthTrack, iXData );
      drawGrid( g, iStartTrack, iWidthTrack, iXData );
    }
  }

  /** Method paint( Graphics g )
   * <p> This method will paint Porosity & Permeability Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
  }
}