/*
 * @ReadLASToolsXMLFile.java Version 1.0 08/30/2004
 *
 * Copyright (c) 2001 John R. Victorine
 * All Rights Reserved.
 */

package xml;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.zip.*;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;

import cmn.cmnString;
import xml.LASToolsXMLStruct;
import cmn.cmnLASToolsListStruct;
import cmn.cmnLASToolsStruct;

/** Class ReadLASToolsXMLFile()
 * <p> This Class will parse the Standard LAS Tools XML Data Stream
 *     directly to the Standard LAS Tools List Data Structure.
 *
 *  @version 1.1 08/30/2004
 *  @author  John R. Victorine
 */

public class ReadLASToolsXMLFile extends DefaultHandler
{
  public static final int FILE       = 0;
  public static final int URL        = 1;
  public static final int SERVER     = 2;
  public static final int SERVER_ZIP = 3;

  private static final int NONE      = -1;
  private static final int TOOL      = 0;
  private static final int OTHER     = 1;

  private int                      iType       = FILE;
  private int                      iXML        = NONE;
  private int                      iRows       = 0;
  private int                      iCount      = -1;
  private int                      iOtherRows  = 0;
  private cmnLASToolsListStruct    stList = new cmnLASToolsListStruct();

  /** Method ReadLASToolsXMLFile()
   *  This is the Default Constructor for this class.
   */

  public ReadLASToolsXMLFile() { iType = FILE; }

  /** Method ReadLASToolsXMLFile()
   *  This is the Constructor for this class.
   *  @param iType = The source identifier,
   *                 0 = File is an absolute Directory Path + Filename
   *                 1 = Source is a URL Location.
   *                 2 = Server Location
   *                 3 = WebStart Location
   */

  public ReadLASToolsXMLFile(int iType) { this.iType = iType; }

  /** Method Process()
   *  <p> This method will Process the XML File into a Array List.
   *      It is a double pass method in that it Reads the XML File
   *      once to get the number of lines in the XML File and a
   *      second time to actually read the XML File.
   *  @param  filename = The File to be opened.
   *  @return stList   = The XML List Data Structure (Array List).
   */

  public cmnLASToolsListStruct Process(String filename)
  {
    stList.iCount = 0;
    Read(filename);
    stList.iCount = iCount+1;

    return (stList);
  }

  /** Method Read()
   *  <p> This method will Read & Parse the XML File.
   *  @param filename = the filename of the XML File.
   */

  public void Read(String filename)
  {
    // Use the validating parser

    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setValidating(true);

    try
    {
      SAXParser saxParser = factory.newSAXParser();
      if (iType == FILE)
      {
        saxParser.parse( new File(filename), this);
      }
      else if (iType == URL)
      {
        try
        {
          InputSource source = new InputSource(filename);
          saxParser.parse(source, this);
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
      else if (iType == SERVER)
      {
        try
        {
          // Connect to the server

          URL u = new URL(filename);
          URLConnection uc = u.openConnection();
          HttpURLConnection connection = (HttpURLConnection) uc;
          connection.setDoOutput(true);
          connection.setDoInput(true);
          connection.setRequestMethod("POST");

          // Read the response XML Document

          InputStream in = connection.getInputStream();
          InputSource source = new InputSource(in);
          saxParser.parse(source, this);
          in.close();
          connection.disconnect();
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
      else
      {
        try
        {
          // Connect to the server

          URL u = new URL(filename);
          URLConnection uc = u.openConnection();
          HttpURLConnection connection = (HttpURLConnection) uc;
          connection.setDoOutput(true);
          connection.setDoInput(true);
          connection.setRequestMethod("POST");

          // Read the response XML Document

          GZIPInputStream in = new GZIPInputStream(connection.getInputStream());
          InputSource source = new InputSource(in);
          saxParser.parse(source, this);
          in.close();
          connection.disconnect();
        }
        catch (Exception e)
        {
          System.err.println(e);
        }
      }
    }
    catch (SAXParseException spe)
    {
      // Error generated by the parser

      System.out.println("\n** Parsing error"            +
                         ", line " + spe.getLineNumber() +
                         ", uri "  + spe.getSystemId());
      System.out.println("   " + spe.getMessage() );

      // Use the contained exception, if any

      Exception  x = spe;
      if (spe.getException() != null)
        x = spe.getException();
      x.printStackTrace();
    }
    catch (SAXException sxe)
    {
      // Error generated by this application (or a parser-initialization error)

      Exception  x = sxe;
      if (sxe.getException() != null)
        x = sxe.getException();
      x.printStackTrace();
    }
    catch (ParserConfigurationException pce)
    {
      // Parser with specified options can't be built

      pce.printStackTrace();
    }
    catch (IOException ioe)
    {
      // I/O error

      ioe.printStackTrace();
    }
  }

  //===========================================================
  // SAX DocumentHandler methods
  //===========================================================

  /** Method setDocumentLocator()
   * <p> This is the Set Document Locator Method
   * @param l Document Locator
   */

  public void setDocumentLocator(Locator l)
  {
    // Ignore errors
  }

  /** Method startDocument()
   * <p> This is the Start Document Method
   * @throws SAXException
   */

  public void startDocument() throws SAXException
  {
    // Ignore it
  }

  /** Method endDocument()
   * <p> This is the End Document Method
   * @throws SAXException
   */

  public void endDocument() throws SAXException
  {
    // Ignore it
  }

  /** Method startElement()
   * <p> This method will parse the ELEMENT from an XML File
   * @param namespaceURI = Name Space URI
   * @param lName        = Local Name
   * @param qName        = Qualified Name
   * @param attrs        = Attributes
   * @throws SAXException
   */

  public void startElement(String namespaceURI,
                           String lName, // local name
                           String qName, // qualified name
                           Attributes attrs) throws SAXException
  {
    String eName = lName; // element name

    if ("".equals(eName))
      eName = qName; // namespaceAware = false

    // If the ELEMENT Well is detected then initialize the structure.

    if (eName.equals(xml.LASToolsXMLStruct.LAS_CURVE))
    {
      iCount++;
      if (iCount < iRows)
      {
        stList.stItem[iCount]           = new cmnLASToolsStruct();
        stList.stItem[iCount].sKEY      = new String("0");
        stList.stItem[iCount].sMnemonic = new String("");
        stList.stItem[iCount].sDesc     = new String("");
        stList.stItem[iCount].sUnit     = new String("");
        stList.stItem[iCount].sLogStand = new String("");
        stList.stItem[iCount].dMinimum  = 0.0;
        stList.stItem[iCount].dMaximum  = 0.0;
        stList.stItem[iCount].dTrack    = 0;

        stList.stItem[iCount].iOther    = 0;
        stList.stItem[iCount].sOther    = new String[200];
      }
    }

    if (eName.equals(xml.LASToolsXMLStruct.TOOL))
    {
      iXML = TOOL;
    }

    if (eName.equals(xml.LASToolsXMLStruct.OTHER))
    {
      iOtherRows = 0;
      iXML       = OTHER;
    }

    // Add Attributes to the Data List Structure

    if (attrs != null)
    {
      for (int i = 0; i < attrs.getLength(); i++)
      {
        String aName = attrs.getLocalName(i); // Attr name
        if ("".equals(aName))
          aName = attrs.getQName(i);

        String sTemp = new String(attrs.getValue(i));
        String sOut  = removeExcess(sTemp);

        // Get the total number of records available

        if (aName.equals(xml.LASToolsXMLStruct.RECORDS))
        {
          if (iXML != OTHER)
          {
            iRows = cmn.cmnString.stringToInt(sOut.trim());

            if (iRows > 0)
            {
              stList.stItem = new cmnLASToolsStruct[iRows];
            }
          }
          else
          {
            iOtherRows = cmn.cmnString.stringToInt(sOut.trim());
          }
        }

        // Parse each attribute and add it to the structure

        if (iCount > -1)
        {
          if (iCount < iRows)
          {
            stList.stItem[iCount] = ParseData(aName,
                                              sOut.trim(),
                                              stList.stItem[iCount]);
          }
        }

        // End Add Attributes to List
      }
    }
  }

  /** Method endElement()
   * <p> This method will parse the ELEMENT from an XML File
   * @param namespaceURI = Name Space URI
   * @param sName        = Simple Name
   * @param qName        = Qualified Name
   * @throws SAXException
   */

  public void endElement(String namespaceURI,
                         String sName, // simple name
                         String qName  // qualified name
                        ) throws SAXException
  {
    // Ignore it
  }

  /** Method characters()
   * <p> This method will parse the CHARACTERS from an XML File
   * @param buf    = Buffer Character Array holding the characters
   * @param offset = String Offset
   * @param len    = Length of String
   * @throws SAXException
   */

  public void characters(char buf[], int offset, int len) throws SAXException
  {
    // Ignore it
  }

  /** Method ignorableWhitespace()
   * <p> This method will parse the Ignorable White Space from an XML File
   * @param buf    = Buffer Character Array holding the characters
   * @param offset = String Offset
   * @param len    = Length of String
   * @throws SAXException
   */

  public void ignorableWhitespace(char buf[], int offset, int len)
    throws SAXException
  {
    // Ignore it
  }

  /** Method processingInstruction()
   * <p> This method will processing Instructions for a XML File
   * @param target = Target
   * @param data   = Data
   * @throws SAXException
   */

  public void processingInstruction(String target, String data)
    throws SAXException
  {
    // Ignore it
  }

  //===========================================================
  // SAX ErrorHandler methods
  //===========================================================

  /** Method error()
   * <p> This method will treat validation errors as fatal
   * @param e = SAX Parse Exception
   * @throws SAXParseException
   */

  public void error(SAXParseException e) throws SAXParseException { throw e; }

  /** Method warning()
   * <p> This method will treat warnings
   * @param err = SAX Parse Exception
   * @throws SAXParseException
   */

  public void warning(SAXParseException err) throws SAXParseException
  {
    System.out.println("** Warning" +
                       ", line "    + err.getLineNumber() +
                       ", uri "     + err.getSystemId());
    System.out.println("   " + err.getMessage());
  }

  //===========================================================
  // Utility Methods ...
  //===========================================================

  /** Method removeExcess()
   * <p> Remove excess white space within a string
   * @param  sin  = String to be parsed of excess spaces
   * @return sout = String with the excess spaces removed
   */

  private String removeExcess(String sin)
  {
    String sout   = new String("");
    char   chold  = ' ';
    char   ch[];

    ch = sin.toCharArray();

    for (int i=0; i<ch.length; i++)
    {
      if (i==0)
        chold = ch[i];

      if (chold != ' ')
      {
        sout = new String(sout + ch[i]);
      }
      else if (chold == ' ')
      {
        if (ch[i] != ' ')
        {
          sout = new String(sout + ch[i]);
        }
      }

      chold = ch[i];
    }

    return (sout);
  }

  /** ParseData()
   * <p> This method will parse the Data Stream for the Information
   * @param sIdentifier = The Attribute Identifier
   * @param  sData      = LAS File String
   * @param  st         = The LAS File Data Structure
   * @return st         = The LAS File Data Structure
   */

  private cmnLASToolsStruct ParseData(String sIdentifier,
                                      String sData,
                                      cmnLASToolsStruct st)
  {
    if (sIdentifier.equals(xml.LASToolsXMLStruct.KEY))        // TOOL KID
    {
      st.sKEY = new String(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.MNEMONIC))   // Curve Mnemonic
    {
      st.sMnemonic = new String(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.DESCRIPTION))// Description
    {
      st.sDesc = new String(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.UNIT))       // Curve Unit
    {
      st.sUnit = new String(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.STANDARD))   // Standard Name
    {
      st.sLogStand = new String(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.MINIMUM))    // Minimum Value
    {
      st.dMinimum = 0.0;
      if (cmn.cmnString.isNumeric(sData))
        st.dMinimum = cmn.cmnString.stringToDouble(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.MAXIMUM))    // Maximum Value
    {
      st.dMaximum = 0.0;
      if (cmn.cmnString.isNumeric(sData))
        st.dMaximum = cmn.cmnString.stringToDouble(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.TRACK))      // Track
    {
      st.dTrack = 0.0;
      if (cmn.cmnString.isNumeric(sData))
        st.dTrack = cmn.cmnString.stringToDouble(sData);
    }

    if (sIdentifier.equals(xml.LASToolsXMLStruct.O_MNEMONIC)) // OTHER Mnemonics
    {
      st.sOther[st.iOther] = new String(sData);
      st.iOther++;
    }

    return (st);
  }
}