/*
 * @xsectionPlotPanel.java Version 1.1 02/20/2008
 *
 * Copyright (c) 2008 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection.gui;

import java.awt.*;
import java.awt.image.*;
import java.util.Observable;
import javax.swing.*;

import cmn.cmnStruct;

import iqstrat.iqstratStruct;           // Global IQSTRAT Data Structure

import xsection.xsectionStruct;         // Global Cross Section Data Structure
import xsection.xsectionDataListStruct; // Cross Section List Structure
import xsection.plot.xsectionPlot;      // Cross Section Plot

/** Class xsectionPlotPanel
 *  <p> This Class will create the stratigraphic Plot.
 *
 *  @version 1.1 02/20/2008
 *  @author  John R. Victorine
 */

public class xsectionPlotPanel extends JPanel
{
  // Input Variables

  private iqstratStruct stStruct   = null; // Global IQSTRAT Data Structure
  private double        depthStart = 0.0;  // Starting Depth
  private double        depthEnd   = 0.0;  // Ending Depth
  private int           iScale     = -1;   // Plot Scale

  // Global Variables

  private int iWidth   = 0;
  private int iHeight  = 0;

  private xsectionPlot plot    = null;

  // Global Panel Widgets

  private ScrollPane scroll = new ScrollPane();
  private Panel      panel  = new Panel();

  private Cursor normalCursor = null;
  private Cursor mouseCursor  = null;

  /** Constructor xsectionPlotPanel( iqstratStruct stStruct,
   *                                 double depthStart,
   *                                 double depthEnd,
   *                                 int    iScale,
   *                                 Cursor normalCursor,
   *                                 Cursor mouseCursor   )
   *  <p> This is the Constructor for this class.
   *  @param stStruct     = Global IQSTRAT Data Structure
   *  @param depthStart   = The starting depth of plot
   *  @param depthEnd     = The ending depth of plot
   *  @param iScale       = The scale
   *  @param normalCursor = Normal Cursor
   *  @param mouseCursor  = Hand Cursor
   */

  public xsectionPlotPanel( iqstratStruct stStruct,
                            double depthStart,
                            double depthEnd,
                            int    iScale,
                            Cursor normalCursor,
                            Cursor mouseCursor )
  {
    try
    {
      this.stStruct     = stStruct;
      this.depthStart   = depthStart;
      this.depthEnd     = depthEnd;
      this.iScale       = iScale;
      this.normalCursor = normalCursor;
      this.mouseCursor  = mouseCursor;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    this.setLayout(new BorderLayout());

    plot    = new xsectionPlot( stStruct, depthStart, depthEnd, iScale );
    iWidth  = plot.getPlotWidth();
    iHeight = plot.getPlotHeight();

    panel.setLayout(new BorderLayout());
    panel.setPreferredSize(new Dimension(iWidth, iHeight));

    // Attach widgets to panel

    this.add(scroll,  BorderLayout.CENTER);
    scroll.add(panel, null);
    panel.add(plot,   BorderLayout.CENTER);
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stStruct  = null;

    // Global Variables

    if (plot != null)
      plot.close();
    plot    = null;

    // Global Panel Widgets

    scroll = null;
    panel  = null;

    normalCursor = null;
    mouseCursor  = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (plot != null) image = plot.getImage();

    return (image);
  }

  /** Method getWell()
   *  <p> This method will identify which well was selected
   *  @return the Well Order Number
   */

  public int getWell() { return (plot.getWell()); }

  /** Method getZeller()
   *  <p> This method will pass the Zeller KID selected
   *  @return the Zeller KID
   */

  public String getZeller() { return (plot.getZeller()); }

  /** Method double getDepth()
   *  <p> This method will pass the depth selected
   *  @return depthSelect = the depth Selected
   */

  public double getDepth() { return (plot.getDepth()); }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /* ------------------------------------------------------------------ *
   * ------------- Set Observable to handle frame actions ------------- *
   * ------------------------------------------------------------------ */

  /** Method setObservable( Observable notifier )
   * <p> This method will set the Observerable Object
   * @param notifier = the observable object
   */

  public void setObservable( Observable notifier )
  {
    if (plot != null) plot.setObservable(notifier);
  }

  /* ------------------------------------------------------------------ *
   * ---------------- Set Global Common Data Structure ---------------- *
   * ------------------------------------------------------------------ */

  /** Method setCommon( cmnStruct stCMN )
   * <p> This method will set the Global Common Data Structure
   *  @param stCMN  = the Global Common Data Structure
   */

  public void setCommon( cmnStruct stCMN )
  {
    if (plot != null) plot.setCommon(stCMN);
  }

  /* ------------------------------------------------------------------ *
   * -------------------------- Depth Scale --------------------------- *
   * ------------------------------------------------------------------ */

  /** Method setWorkingDepth()
   *  <p> This method will set the Working Depth()
   *  @param iEdit    = Set the Set Working Depth Indicator
   *  @param depthTop = Set the Working Top Depth
   */

  public void setWorkingDepth(int iEdit, double depthTop)
  {
	if (plot != null) plot.setWorkingDepth( iEdit, depthTop );
  }

  /** Method setCopyDepth()
   *  <p> This method will set the plot to show the copy &
   *      paste tops information
   *  @param iCopy    = Set the Copy Indicator
   *  @param sZeller  = Set the Zeller Tops KID
   *  @param sName    = Set the Name of the Top
   */

  public void setCopyDepth(int iCopy, String sZeller, String sName)
  {
	if (plot != null) plot.setCopyDepth( iCopy, sZeller, sName );
  }

  /** Method setEvalDepth()
   *  <p> This method will set the Working Depth()
   *  @param iEval     = Set the Set Evaluator Depth Indicator
   *  @param depthEval = Set the evaluator Top Depth
   */

  public void setEvalDepth(int iEval, double depthEval)
  {
	if (plot != null) plot.setEvalDepth( iEval, depthEval );
  }

  /** Method setScale( int iScale, double dMaximum, double dMinimum )
   * <p> This method will set the Plot scale in the
   *     Cross Plot Data List Structure
   * @param iScale   = The Depth Track Scale identifier
   * @param dMaximum = The Maximum Depth Value
   * @param dMinimum = The Minimum Depth Value
   */

  public void setScale( int iScale, double dMaximum, double dMinimum )
  {
    if (plot != null)
    {
      plot.setScale( iScale, dMaximum, dMinimum );
      iWidth  = 50 + plot.getPlotWidth();
      iHeight = plot.getPlotHeight();
      panel.setPreferredSize( new Dimension( iWidth, iHeight ) );
    }

    scroll.doLayout();
    panel.doLayout();
  }

  /* ------------------------------------------------------------------ *
   * --------------------------- Texfields ---------------------------- *
   * ------------------------------------------------------------------ */

  /** Method setTextfields()
   * <p> This method will set the label widgets so the location of the
   *     mouse cursor can be described on the Control Panel
   * @param lblTops  = Label that will hold the name of well the cursor
   *                   is moving over.
   * @param lblDepth = Label that will hold the actual LAS or Core Depth
   * @param lblElev  = Label that will hold the Depth at elevation.
   *

  public void setTextfields( JLabel lblTops[], JLabel lblDepth[], JLabel lblElev[] )
  {
    if (plot != null)
      plot.setTextfields( lblTops, lblDepth, lblElev );
  }
*/
  /** Method setTextfields()
   * <p> This method will set the label widgets so the location of the
   *     mouse cursor can be described on the Control Panel
   * @param lblOrder = Label that will hold the Well Order Number
   * @param lblName  = Label that will hold the Well Name
   * @param lblAPI   = Label that will hold the Well API-Number
   * @param lblTops  = Label that will hold the name of well the cursor
   *                   is moving over.
   * @param lblDepth = Label that will hold the actual LAS or Core Depth
   * @param lblElev  = Label that will hold the Depth at elevation.
   */

  public void setTextfields( JLabel lblOrder, JLabel lblName,  JLabel lblAPI,
                             JLabel lblTops,  JLabel lblDepth, JLabel lblElev )
  {
    if (plot != null)
      plot.setTextfields( lblOrder, lblName, lblAPI, lblTops, lblDepth, lblElev );
  }

  /* ------------------------------------------------------------------ *
   * ----------------------- Cross Section Data ----------------------- *
   * ------------------------------------------------------------------ */

  /** Method setData( xsectionDataListStruct stXList )
   * <p> This method will set the cross section data
   * @param stXList = Cross Section Data
   */

  public void setData( xsectionDataListStruct stXList )
  {
    if (plot != null)
    {
      plot.setData( stXList );
      iWidth  = 50 + plot.getPlotWidth();
      iHeight = plot.getPlotHeight();
      panel.setPreferredSize( new Dimension( iWidth, iHeight ) );
    }

    scroll.doLayout();
    panel.doLayout();
  }

  /** Method setLASTrackSize(int iLASSize[])
   * <p> This method will set the LAS File Track Width
   * @param iLASSize = Array of LAS File Track Width
   */

  public void setLASTrackSize(int iLASSize[])
  {
    if (plot != null)  plot.setLASTrackSize( iLASSize );
  }

  /** Method setDatum( int iDatum, String sDatum  )
   * <p> This method will set the datum by identifier button
   * @param iDatum = datum by identifier
   * @param sDatum = horizon name
   */

  public void setDatum( int iDatum, String sDatum )
  {
    if (plot != null)
      plot.setDatum( iDatum, sDatum );
  }

  /** Method setSelected()
   * <p> This method will change the selected tracks that are plotted
   *  @param iWell     = The well order number
   *  @param iSelected = The Tracks that will be displayed in the selected well
   */

  public void setSelected( int iWell, int iSelected[] )
  {
	if (plot != null) { plot.setSelected( iWell, iSelected ); }
  }

  /** Method setLevel( int iLevel )
   *  <p> This method will load the indicator to turn on the confidence
   *      level color map iLevel=1 or turn off iLevel=0. Default is 0
   *  @param iLevel = the confidence level plot switch
   */

  public void setLevel(int iLevel )
  {
	if (plot != null){ plot.setLevel(iLevel); }
  }

  /** Method refresh()
   * <p> This method will set the ICS stratigraphic Units List Structure
   */

  public void refresh() { if (plot != null) plot.refresh(); }

}