/*
 * @xsectionTypeLogTopsFrame.java Version 1.1 11/08/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package xsection.gui;

import java.util.Observer;
import java.util.Observable;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import iqstrat.iqstratStruct;                 // Common IQSTRAT Data Structure

import kgs.kgsUserStruct;                     // KGS User Data Structure

import horizon.strat.stratListStruct;         // Stratigraphic Units List Structure
import horizon.strat.stratStruct;             // Stratigraphic Units Data Structure
import horizon.strat.gui.stratPlotStateFrame; // Strat Unit Chart Dialog
import horizon.strat.gui.stratListFrame;      // Strat Unit Table Dialog

import xsection.xsectionDataListStruct;       // Cross Section List Structure

/** Class xsectionTypeLogTopsFrame
 *  <p> This Class will allow the user to add, modify and delete Type Log
 *      Tops to the Edit Well.  To view tops information on the reference
 *      well and to transfer the tops information to the edit well.
 *
 *  @version 1.1 11/08/2012
 *  @author  John R. Victorine
 */

public class xsectionTypeLogTopsFrame extends JFrame implements ActionListener, Observer
{
  // Input Variables

  private Observable    pNotifier = null; // Observable Object for calling Frame
  private iqstratStruct stStruct  = null; // Common IQSTRAT Data Structure

  // Global Variables

  private Observable    notifier = null; // Observable Object for this Frame

  private int    iLevel = -1;
  private String sTitle = "";
  private String sTops  = "";
  private String source = "";

  // Log Depth Range and Step

  private double depthStart = 0.0;    // Start log depth of Well/Outcrop
  private double depthStop  = 1000.0; // Stop log depth of Well/Outcrop
  private double depthStep  = 0.5;    // Step By log depth of Well/Outcrop

  // Horizons Depth Start & End

  private double dStart     = 0.0;    // Horizon Top
  private double dEnd       = 0.0;    // Horizon Base

  private double dEval      = 0.0;    // Evaluator Depth

  private int    iWell      = -1;     // Well Number
  private String sWell      = "";     // Well KEY
  private String src_kid    = "";     // Source KEY
  private String sZeller    = "0";    // Zeller KEY

  private kgsUserStruct          stUser  = null; // User Information Data Structure
  private stratStruct            stStrat = null; // Stratigraphic Units Data Structure
  private stratStruct            stTemp  = null; // Temporary Stratigraphic Units Data Structure
  private stratListStruct        stEval  = null; // Evaluated Stratigraphic Units Data Structure
  private xsectionDataListStruct stList  = null; // Cross Section List Structure

  private stratPlotStateFrame    pChart  = null; // Kansas Strat Units Chart Dialog
  private stratListFrame         pTable  = null; // Kansas Strat Units Table Dialog

  // Global Widgets

  private JLabel     lblLevel  = new JLabel(); // Level of Confidence
  private JLabel     lblSource = new JLabel(); // Horizon's Source

  // -- Search Horizon Name By

  private JButton    btnTable = new JButton();  // Search By Table
  private JButton    btnChart = new JButton();  // Search By Chart

  // -- Horizon Name

  private JTextField txtTops  = new JTextField(); // Horizon Name

  // -- Horizons Depth Range

  private JSpinner spinnerStart = null;  // Depth Top
  private JSpinner spinnerEnd   = null;  // Depth Base

  // -- Refresh Working Top Depth

  private JButton    btnRefresh = new JButton();  // Refresh Working Depth

  // -- Icon Buttons

  private JButton    btnCopy    = null; // Copy Top Data
  private JButton    btnDatumBy = null; // Datum By Top Data

  private JButton    btnSave    = null; // Save Tops Data
  private JButton    btnDelete  = null; // Delete Tops Data

  private JButton    btnAccept  = null; // Accept Tops Data
  private JButton    btnReject  = null; // Reject Tops Data

  private JButton      btnIcons[]          = null; // Evaluated Source Data
  private TitledBorder titledBorderIcons[] = null; // Titled Border for Evaluated Icons

  private JLabel     lblNote    = new JLabel(); // Note to User

  /** Constructor xsectionTypeLogTopsFrame( Observable pNotifier )
   *  <p> This is the Constructor for this class.
   *  @param pNotifier = Observable
   *  @param stStruct  = Common IQSTRAT Data Structure
   *  @param stList    = Cross Section List Structure
   *  @param iWell     = Well Selected
   *  @param sZeller   = The Zeller ID Number
   *  @param depth     = the depth clicked
   */

  public xsectionTypeLogTopsFrame( Observable pNotifier,
                                   iqstratStruct stStruct,
                                   xsectionDataListStruct stList,
                                   int iWell, String sZeller, double depth )
  {
	stratListStruct st       = null;
    int             iCount   = 0;
    int             iPercent = 0;

    try
    {
      this.pNotifier = pNotifier;
      this.stStruct  = stStruct;
      this.stList    = stList;
      this.iWell     = iWell;
      this.sZeller   = sZeller;
//System.out.println("IN "+iWell+" <"+sZeller+">");

      if (stStruct.stCMN.stUser == null)
        stStruct.stCMN.stUser = new kgsUserStruct();

      source  = new String(stStruct.stCMN.stUser.sName+" ("+stStruct.stCMN.stUser.sAccount+")");
      src_kid = new String( stStruct.stCMN.stUser.sKEY );
      iLevel  = 3;

      if (iWell > -1)
      {
		sTitle = new String( stList.stItem[iWell].stHeader.sName );
		sWell  = new String( stList.stItem[iWell].stHeader.sWELLKID );
	    st     = kgs.io.kgsIORead.ReadWellRegions2( sZeller, sWell,
	                                                horizon.strat.io.ReadStratUnitsXMLFile.URL);

	    stEval = kgs.io.kgsIORead.ReadWellEvalRegions2( sZeller, sWell,
	                                                    horizon.strat.io.ReadStratUnitsXMLFile.URL);

		depthStart = 0.0; //stList.stItem[iWell].stLAS.depthStart;
		depthStop  = stList.stItem[iWell].stLAS.depthEnd;
		depthStep  = 0.5;
//System.out.println("WELL: "+sWell+" "+depthStart+" "+depthStop);

		dStart  = depth;
		dEnd    = 0.0;

	    iLevel  = 3;
        if (stEval != null)
        {
		  for (int i=0; i<stEval.iCount; i++)
		  {
			if (stEval.stItem[i].sEvaluated.equals("ACCEPT"))
			  iCount++;
		  }

          iPercent = ( iCount * 100 ) / horizon.strat.stratStruct._EVALUATORS;

          if (iPercent >= horizon.strat.stratStruct._MIN_Cu)
            iLevel = 2;

          if (iPercent >= horizon.strat.stratStruct._MIN_Ag)
            iLevel = 1;

          if (iPercent >= horizon.strat.stratStruct._MIN_Au)
            iLevel = 0;
   		}

	    if (stStruct.iCopy == 1)
	      sTops   = new String( stStruct.sTopsName );

        // Clear Temporary Fields

        stStruct.iCopy     = 0;
        stStruct.sTopsName = new String("");

        // Check Well Strat Units List Structure for Tops Information

        if (st != null)
        {
		  if (st.iCount > 0)
		  {
		    stStrat    = horizon.strat.stratUtility.copy( st.stItem[0] );
		  }

		  if (stStrat != null)
		  {
			sTops   = new String( stStrat.sName );

			dStart  = stStrat.depthStart;

			if (dStart < depthStart)
			  dStart = depthStart;
			if (dStart > depthStop)
			  dStart = depthStop;

			dEnd    = stStrat.depthEnd;

			if (dEnd < dStart)
			  dEnd = dStart;

			if (dEnd < depthStart)
			  dEnd = depthStart;
			if (dEnd > depthStop)
			  dEnd = depthStop;
//System.out.println("TOPS: "+sTops+" "+dStart+" "+dEnd);

            stUser = kgs.io.kgsIOUser.getSource( stStrat.source );
            if (stUser != null)
            {
			  src_kid = new String( stStrat.source );
              source  = new String( stUser.sName+" ("+stUser.sAccount+")" );
//System.out.println("SOURCE: "+src_kid);
			}
		  }
	    }
	  }

      jbInit();

      addWindowListener( new xsectionTypeLogTopsFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    kgsUserStruct stUserEval = null; // User Information Data Structure

    JPanel      pnlTop     = new JPanel();
    JPanel      pnlTop1    = new JPanel();
    JPanel      pnlLevel   = new JPanel();
    JPanel      pnlSource  = new JPanel();
    JPanel      pnlChart   = new JPanel();
    JPanel      pnlHorizon = new JPanel();

    JPanel      pnlCenter  = new JPanel();
    JPanel      pnlCenterW = new JPanel();
    JPanel      pnlDepths  = new JPanel();
    JPanel      pnlRange   = new JPanel();
    JPanel      pnlStart   = new JPanel();
    JPanel      pnlEnd     = new JPanel();

    JPanel      pnlBase    = new JPanel();
    JPanel      pnlIcon    = new JPanel();
    JPanel      pnlCopy    = new JPanel();
    JPanel      pnlDatumBy = new JPanel();
    JPanel      pnlSave    = new JPanel();
    JPanel      pnlDelete  = new JPanel();
    JPanel      pnlAccept  = new JPanel();
    JPanel      pnlReject  = new JPanel();

    JPanel      pnlEval    = new JPanel();
    JPanel      pnlEvalIcons[] = null;
    String      sEvalUsers[]   = null;

    TitledBorder titledBorderLevel = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Confidence:");
    titledBorderLevel.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderSource = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Horizon Source:");
    titledBorderSource.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderChart = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Find Horizon Name By:");
    titledBorderChart.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHorizon = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Horizon (Ignore rank terms System, Group, Formation, etc.)");
    titledBorderHorizon.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderRange = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Depth Range");
    titledBorderRange.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderStart = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Top:");
    titledBorderStart.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEnd = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Base:");
    titledBorderEnd.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderCopy = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Copy");
    titledBorderCopy.setTitleJustification(TitledBorder.CENTER);
    titledBorderCopy.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDatumBy = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "DatumBy");
    titledBorderDatumBy.setTitleJustification(TitledBorder.CENTER);
    titledBorderDatumBy.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderSave = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Save");
    titledBorderSave.setTitleJustification(TitledBorder.CENTER);
    titledBorderSave.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDelete = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Delete");
    titledBorderDelete.setTitleJustification(TitledBorder.CENTER);
    titledBorderDelete.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderAccept = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Accept");
    titledBorderAccept.setTitleJustification(TitledBorder.CENTER);
    titledBorderAccept.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderReject = new TitledBorder(
        BorderFactory.createEmptyBorder(),  "Reject");
    titledBorderReject.setTitleJustification(TitledBorder.CENTER);
    titledBorderReject.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Create an Observable

    notifier = new xsectionTypeLogTopsFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver( this );

    // Build Frame

    this.setTitle( sTitle );
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // Top 1 Panel

    pnlTop1.setLayout(new BorderLayout());

    // .. Level of Confidence Panel

//    pnlLevel.setBorder(titledBorderLevel);
    pnlLevel.setBorder(BorderFactory.createEtchedBorder());
    pnlLevel.setLayout(new BorderLayout());
    pnlLevel.setPreferredSize( new Dimension(140, 10) );

    if (iLevel == -1)
    {
	  lblLevel.setText( "" );
	}
	else
	{
      pnlLevel.setBackground( horizon.strat.stratStruct.COLORS[iLevel] );

      lblLevel.setBackground( horizon.strat.stratStruct.COLORS[iLevel] );
      lblLevel.setText( " Level: " + horizon.strat.stratStruct.LEVEL[iLevel] + " " );
	}

    lblLevel.setHorizontalAlignment(SwingConstants.CENTER);
//    lblLevel.setVerticalAlignment(SwingConstants.BOTTOM);
    lblLevel.setFont(new java.awt.Font("Dialog", 1, 11));
    if (iLevel == 3)
      lblLevel.setForeground( Color.yellow );
    else
      lblLevel.setForeground( Color.black );

    // .. Horizon's Source Panel

    pnlSource.setLayout(new BorderLayout());
    pnlSource.setBorder(titledBorderSource);

    lblSource.setText( source );
    lblSource.setHorizontalAlignment(SwingConstants.CENTER);
    lblSource.setFont(new java.awt.Font("Dialog", 1, 11));

    // .. Search Horizon Name By Panel

    pnlChart.setBorder(titledBorderChart);
    pnlChart.setLayout(new GridLayout());

    btnTable.setFont(new java.awt.Font("Dialog", 1, 11));
    btnTable.setText("Table");
    btnTable.setEnabled( false );
    btnTable.addActionListener(this);

    btnChart.setFont(new java.awt.Font("Dialog", 1, 11));
    btnChart.setText("Chart");
    btnChart.setEnabled( false );
    btnChart.addActionListener(this);

    if ((iWell > 0) && (stStruct.stCMN.stUser.sKEY.equals(src_kid)))
    {
      btnTable.setEnabled( true );
      btnChart.setEnabled( true );
	}

    // -- Horizon Panel

    pnlHorizon.setLayout(new BorderLayout());
    pnlHorizon.setBorder(titledBorderHorizon);

    txtTops.setText( sTops );
    txtTops.setFont(new java.awt.Font("Dialog", 1, 11));
    txtTops.setEditable(false);
    if ((iWell > 0) && (stStruct.stCMN.stUser.sKEY.equals(src_kid)))
      txtTops.setEditable(true);
    txtTops.addFocusListener(new xsectionTypeLogTopsFrameFocusAdapter(this));

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Center Note Panel

    if ((iWell > 0) && (stStruct.stCMN.stUser.sKEY.equals(src_kid)))
      lblNote.setText("Before you Save, Set Your Top Depth ( Base Depth can be 0 )");
    else if ((iWell > 0) && (!stStruct.stCMN.stUser.sKEY.equals(src_kid)))
      lblNote.setText("Before you Reject, Set Your Top & Base Depths");
    else
      lblNote.setText("");
    lblNote.setHorizontalAlignment(SwingConstants.CENTER);
    lblNote.setFont(new java.awt.Font("Dialog", 1, 13));
    lblNote.setForeground( Color.red );

    // -- Center West Panel

    pnlCenterW.setLayout(new BorderLayout());

    // -- Evaluated Tops Icon Panel

    pnlEval.setLayout(new GridLayout());

    if ((stEval != null) && (iWell > 0))
    {
	  sEvalUsers        = new String[stEval.iCount];
      pnlEvalIcons      = new JPanel[stEval.iCount];
      btnIcons          = new JButton[stEval.iCount];
      titledBorderIcons = new TitledBorder[stEval.iCount];

	  for (int i=0; i<stEval.iCount; i++)
	  {
        stUserEval = kgs.io.kgsIOUser.getSource( stEval.stItem[i].source );
        if (stUserEval != null)
        {
		  sEvalUsers[i]        = new String(stUserEval.sAccount);
          titledBorderIcons[i] = new TitledBorder(
            BorderFactory.createEmptyBorder(),  sEvalUsers[i]);
          titledBorderIcons[i].setTitleJustification(TitledBorder.CENTER);
          titledBorderIcons[i].setTitleFont(new java.awt.Font("Dialog", 1, 11));
	    }

		pnlEvalIcons[i] = new JPanel();
        if (stUserEval != null)
          pnlEvalIcons[i].setBorder( titledBorderIcons[i] );
        pnlEvalIcons[i].setBackground(Color.white);

        if (stEval.stItem[i].sEvaluated.equals("ACCEPT"))
          btnIcons[i] = new JButton( createImageIcon( cmn.cmnStruct.ACCEPT ) );
        else
          btnIcons[i] = new JButton( createImageIcon( cmn.cmnStruct.REJECT ) );
        btnIcons[i].setBackground(Color.white);
        btnIcons[i].setPreferredSize( new Dimension(50, 50) );
        btnIcons[i].addActionListener( this );

        pnlEval.add(pnlEvalIcons[i],     null);
        pnlEvalIcons[i].add(btnIcons[i], null);
	  }
	}

    // -- Depths Panel

    pnlDepths.setLayout(new BorderLayout());

    // .. Depth Range Panel

    pnlRange.setLayout(new GridLayout());
    pnlRange.setBorder(titledBorderRange);

    ChangeListener listener = new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        SpinnerModel source = (SpinnerModel)e.getSource();

        String sTemp = spinnerStart.getValue().toString();
        dStart       = cmn.cmnString.stringToDouble(sTemp);

        sTemp        = spinnerEnd.getValue().toString();
        dEnd         = cmn.cmnString.stringToDouble(sTemp);
      }
    };

    // .... Starting Depth Panel

    pnlStart.setLayout(new BorderLayout());
    pnlStart.setBorder(titledBorderStart);
//System.out.println(dStart+" "+dEnd+" "+depthStart+" "+depthStop+" "+depthStep);

    SpinnerNumberModel modelStart =
        new SpinnerNumberModel(dStart, depthStart, depthStop, depthStep);
    spinnerStart = new JSpinner(modelStart);
    spinnerStart.setBorder(BorderFactory.createRaisedBevelBorder());
    modelStart.addChangeListener(listener);

    // .... Ending Depth Panel

    pnlEnd.setLayout(new BorderLayout());
    pnlEnd.setBorder(titledBorderEnd);

    SpinnerNumberModel modelEnd =
        new SpinnerNumberModel(dEnd, 0.0, depthStop, depthStep);
    spinnerEnd = new JSpinner(modelEnd);
    spinnerEnd.setBorder(BorderFactory.createRaisedBevelBorder());
    modelEnd.addChangeListener(listener);

    // .... Refresh Working Depth

    btnRefresh.setFont(new java.awt.Font("Dialog", 1, 11));
    btnRefresh.setText("Refresh Plot");
    btnRefresh.addActionListener(this);

    // Base Panel

    pnlBase.setLayout(new BorderLayout());

    // .. Icon Button Panel

    pnlIcon.setLayout( new GridLayout());

    // .... Copy Panel

    pnlCopy.setBorder( titledBorderCopy );
    pnlCopy.setBackground(Color.white);

    btnCopy = new JButton( createImageIcon( cmn.cmnStruct.COPY ) );
    btnCopy.setBackground(Color.white);
    btnCopy.setPreferredSize( new Dimension(50, 50) );
    btnCopy.addActionListener( this );

    // .... DatumBy Panel

    pnlDatumBy.setBorder( titledBorderDatumBy );
    pnlDatumBy.setBackground(Color.white);

    btnDatumBy = new JButton( createImageIcon( cmn.cmnStruct.DATUMBY ) );
    btnDatumBy.setBackground(Color.white);
    btnDatumBy.setPreferredSize( new Dimension(50, 50) );
    btnDatumBy.addActionListener( this );

    // .... Save Panel

    pnlSave.setBorder( titledBorderSave );
    pnlSave.setBackground(Color.white);

    btnSave = new JButton( createImageIcon( cmn.cmnStruct.SAVE ) );
    btnSave.setBackground(Color.white);
    btnSave.setPreferredSize( new Dimension(50, 50) );
    btnSave.addActionListener( this );

    // .... Delete Panel

    pnlDelete.setBorder( titledBorderDelete );
    pnlDelete.setBackground(Color.white);

    btnDelete = new JButton( createImageIcon( cmn.cmnStruct.DELETE ) );
    btnDelete.setBackground(Color.white);
    btnDelete.setPreferredSize( new Dimension(50, 50) );
    btnDelete.addActionListener( this );

    // .... Accept Panel

    pnlAccept.setBorder( titledBorderAccept );
    pnlAccept.setBackground(Color.white);

    btnAccept = new JButton( createImageIcon( cmn.cmnStruct.ACCEPT ) );
    btnAccept.setBackground(Color.white);
    btnAccept.setPreferredSize( new Dimension(50, 50) );
    btnAccept.addActionListener( this );

    // .... Reject Panel

    pnlReject.setBorder( titledBorderReject );
    pnlReject.setBackground(Color.white);

    btnReject = new JButton( createImageIcon( cmn.cmnStruct.REJECT ) );
    btnReject.setBackground(Color.white);
    btnReject.setPreferredSize( new Dimension(50, 50) );
    btnReject.addActionListener( this );

    // Attach Widgets to Frame

    this.getContentPane().add( pnlTop,    BorderLayout.NORTH);
    pnlTop.add( pnlTop1,                  BorderLayout.NORTH );
    pnlTop1.add( pnlLevel,                BorderLayout.WEST );
    pnlLevel.add( lblLevel,               BorderLayout.CENTER); //null );

    pnlTop1.add( pnlSource,               BorderLayout.CENTER );
    pnlSource.add( lblSource,             null );

    pnlTop1.add( pnlChart,                BorderLayout.EAST );
    pnlChart.add( btnTable,               null );
    pnlChart.add( btnChart,               null );

    pnlTop.add(pnlHorizon,                BorderLayout.CENTER );
    pnlHorizon.add(txtTops,               BorderLayout.CENTER );

    this.getContentPane().add( pnlCenter, BorderLayout.CENTER );
    pnlCenter.add( pnlCenterW,            BorderLayout.NORTH );

    pnlCenterW.add( pnlEval,              BorderLayout.CENTER );

    if (iWell > 0)
    {
//      pnlCenterW.add( pnlRange,           BorderLayout.WEST );
      pnlCenterW.add( pnlDepths,          BorderLayout.WEST );
      pnlDepths.add( pnlRange,            BorderLayout.CENTER );
      pnlRange.add( pnlStart,             null );
      pnlStart.add( spinnerStart,         BorderLayout.CENTER );

      pnlRange.add( pnlEnd,               null );
      pnlEnd.add( spinnerEnd,             BorderLayout.CENTER );

      pnlDepths.add( btnRefresh,          BorderLayout.SOUTH );
	}

//    if (iWell > 0)
      pnlCenter.add( lblNote,             BorderLayout.SOUTH );

    this.getContentPane().add( pnlBase,   BorderLayout.SOUTH );
    pnlBase.add( pnlIcon,                 BorderLayout.CENTER );

    pnlIcon.add( pnlDatumBy,              null );
    pnlDatumBy.add( btnDatumBy,           null );
/*
    if (iWell == 0)
    {
      pnlIcon.add( pnlCopy,               null );
      pnlCopy.add( btnCopy,               null );
    }

    if ((iWell > 0) && (stStruct.stCMN.stUser.sKEY.equals(src_kid)))
    {
      pnlIcon.add( pnlSave,               null );
      pnlSave.add( btnSave,               null );

      pnlIcon.add( pnlDelete,             null );
      pnlDelete.add( btnDelete,           null );
    }
    else if ((iWell > 0) && (!stStruct.stCMN.stUser.sKEY.equals(src_kid)))
    {
      pnlIcon.add( pnlAccept,             null );
      pnlAccept.add( btnAccept,           null );

      pnlIcon.add( pnlReject,             null );
      pnlReject.add( btnReject,           null );
	}
*/
    // Set the Size and Display the Frame

    if (iWell > 0)
      this.setSize(550, 360);
    else
      this.setSize(550, 240);

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    this.setLocation((d.width  - this.getSize().width)/2,
                     (d.height - this.getSize().height)/2);

    this.setResizable(false);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /** METHOD xsectionTypeLogTopsFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class xsectionTypeLogTopsFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will set the global variables to null in order to force
   *     JAVA to free the memory allocated for this panel.
   */

  public void close()
  {
    if (pNotifier != null)
      pNotifier.notifyObservers(new String("Free Working Depth"));

    // Input Variables

    pNotifier = null; // Observable Object calling frame

    // Global Variables

    notifier  = null; // Observable Object this frame

    sTitle    = null; // Title
    sTops     = null; // Horizon Name
    source    = null; // Source Name

    sWell     = null; // Well Header KID
    sZeller   = null; // Zeller KID
    src_kid   = null; // Source KID

    if (stUser != null)
      stUser.delete();
    stUser  = null; // User Information Data Structure

    if (stStrat != null)
      stStrat.delete();
    stStrat  = null;

    if (stTemp != null)
      stTemp.delete();
    stTemp  = null;

    stList    = null; // Cross Section List Structure

    if (pChart != null)
      pChart.close();
    pChart = null;

    if (pTable != null)
      pTable.close();
    pTable = null;

    // -- Level of Confidence

    lblLevel  = null;

    // -- Horizon's Source

    lblSource = null;

    // -- Search Horizon Name By

    btnTable  = null; // Search By Table
    btnChart  = null; // Search By Chart

    // -- Horizon Name

    txtTops   = null; // Horizon Name

    // -- Horizons Depth Range

    spinnerStart = null;  // Depth Top
    spinnerEnd   = null;  // Depth Base

    // -- Icon Buttons

    btnCopy      = null;  // Copy Top Data
    btnDatumBy   = null;  // Datum By Top Data
    btnSave      = null;  // Save Tops Data
    btnDelete    = null;  // Delete Tops Data
    btnAccept    = null;  // Accept Tops Data
    btnReject    = null;  // Reject Tops Data

    if (stEval != null)
    {
	  for (int i=0; i<stEval.iCount; i++)
	  {
  	    if (btnIcons != null)
          btnIcons[i]          = null; // Evaluated Source Data
  	    if (titledBorderIcons != null)
          titledBorderIcons[i] = null; // Titled Border for Evaluated Icons
	  }

      stEval.delete();
	}
    stEval            = null; // Evaluated Stratigraphic Units Data Structure
    btnIcons          = null; // Evaluated Source Data
    titledBorderIcons = null; // Titled Border for Evaluated Icons

    lblNote           = null; // Note to users

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getDatumBy()
   *  <p> This method will force a daum by action on the log data.
   */

  public String getDatumBy() { return (sTops); }

  /** Method getSelection()
   * <p> This method will retrieve and load the State Stratigraphic data
   *     structure
   */

  public void getSelection()
  {
	int iCont = 1;
   	stratListStruct st = null;

    if (pChart != null)
    {
      stTemp = horizon.strat.stratUtility.copy( pChart.getData() );

      if (stTemp != null)
      {
  	    st = kgs.io.kgsIORead.ReadWellRegions2(
		            stTemp.sKEY,
		            stList.stItem[1].stHeader.sWELLKID,
 	                horizon.strat.io.ReadStratUnitsXMLFile.URL);

 	    if (st != null)
 	    {
		  if (st.iCount > 0)
		  {
		    if (!st.stItem[0].source.equals(source))
		    {
              sZeller = new String( stTemp.sKEY ); // Zeller KID
		      dStart  = st.stItem[0].depthStart;   // Depth of Horizon
		      iCont   = 0;
		    }
		  }
		}
  	  }

      if (iCont == 1)
      {
        setSelection(stTemp);

        pChart.setVisible(false);
	  }
	  else
	  {
  	    if (pNotifier != null)
	       pNotifier.notifyObservers(new String("Not Owner"));
	  }
    }
  }

  /** Method getTableSelection()
   * <p> This method will retrieve and load the State Stratigraphic data
   *     structure
   */

  public void getTableSelection()
  {
	int iCont = 1;
   	stratListStruct st = null;

    if (pTable != null)
    {
      stTemp = horizon.strat.stratUtility.copy( pTable.getData() );

      if (stTemp != null)
      {
  	    st = kgs.io.kgsIORead.ReadWellRegions2(
		            stTemp.sKEY,
		            stList.stItem[1].stHeader.sWELLKID,
 	                horizon.strat.io.ReadStratUnitsXMLFile.URL);

 	    if (st != null)
 	    {
		  if (st.iCount > 0)
		  {
		    if (!st.stItem[0].source.equals(source))
		    {
              sZeller = new String( stTemp.sKEY ); // Zeller KID
		      dStart  = st.stItem[0].depthStart;   // Depth of Horizon
		      iCont   = 0;
		    }
		  }
		}
  	  }

      if (iCont == 1)
      {
        setSelection(stTemp);

        pTable.close();
	  }
	  else
	  {
  	    if (pNotifier != null)
	       pNotifier.notifyObservers(new String("Not Owner"));
	  }
    }
  }

  /** Method getWorkingDepth()
   *  <p> This method will get the Working Depth
   *  @return dStart = Working Top Depth
   */

  public double getWorkingDepth() { return (dStart); }

  /** Method getZellerKID()
   *  <p> This method will get the Zeller KID
   *  @return sZeller = Zeller KID
   */

  public String getZellerKID() { return (sZeller); }

  /** Method getName()
   *  <p> This method will get the Tops Name
   *  @return sName = Tops Name
   */

  public String getName() { return (txtTops.getText()); }

  /** Method getEvalDepth()
   *  <p> This method will get the Evaluator Depth
   *  @return dEval = Working Top Depth
   */

  public double getEvalDepth() { return (dEval); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setSelection()
   *  <p> This method checks the selected top and verifies it is a new top
   *      and not an exisiting top that already has been selected for this
   *      well.
   *  @param st = the select stratigraphic units data structure
   */

  private void setSelection(stratStruct st)
  {
	if (st != null)
	{
      sZeller = new String( st.sKEY ); // Zeller KID
      txtTops.setText( st.sName );

//System.out.println(st.sKID+" "+st.sName);
	}
  }

  /** Method setNote()
   *  <p> This method will set a note to the user to click on the edit
   *      well profile to transfer top.
   */

  public void setNote()
  {
    lblNote.setText("Click on Edit Well Profile Plot to transfer Horizon.");
  }

  /* ===================================================================== *
   * ---------------------- ICON BUTTON ACTIONS -------------------------- *
   * ===================================================================== */

  /** Method ModifySourceTop()
   *  This method will insert/update the top to the Type Log Tops DB Table
   */

  private void ModifySourceTop()
  {
	int    iError = 0;
	String sError = "";
	String sUser  = stStruct.stCMN.stUser.sKEY;
	String sName  = txtTops.getText();

	if (sWell.length() < 2)
	{
	  iError = 1;
	  sError = "There is NO Well Information!";
	}
	else if (sUser.length() < 2)
	{
	  iError = 1;
	  sError = "Problem User ID is NOT Present!";
	}
	else if (sZeller.length() < 2)
	{
	  iError = 1;
	  sError = "You must select a valid Zeller Top! \n" +
	           "Select Table or Chart to select a valid Zeller Top";
	}

    if ((dEnd < dStart) && (dEnd > 0.0))
      dEnd = dStart;

    if (iError == 1)
    {
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    }
    else
    {
	  kgs.io.kgsIOWrite.SaveTopsSourceData( sWell, sUser, sZeller, sName, dStart, dEnd );

	  if (pNotifier != null)
	     pNotifier.notifyObservers(new String("Source Tops Modified"));

	}
  }

  /** Method DeleteSourceTop()
   *  This method will delete the top from the Type Log Tops DB Table
   */

  private void DeleteSourceTop()
  {
	int    iError = 0;
	String sError = "";
	String sUser  = stStruct.stCMN.stUser.sKEY;
	String sName  = txtTops.getText();

	if (sWell.length() < 2)
	{
	  iError = 1;
	  sError = "There is NO Well Information!";
	}
	else if (sUser.length() < 2)
	{
	  iError = 1;
	  sError = "Problem User ID is NOT Present!";
	}
	else if (sZeller.length() < 2)
	{
	  iError = 1;
	  sError = "You must select a valid Zeller Top! \n" +
	           "Select Table or Chart to select a valid Zeller Top";
	}

    if ((dEnd < dStart) && (dEnd > 0.0))
      dEnd = dStart;

    if (iError == 1)
    {
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    }
    else
    {
	  kgs.io.kgsIOWrite.DeleteTopsSourceData( sWell, sUser, sZeller );

	  if (pNotifier != null)
	     pNotifier.notifyObservers(new String("Source Tops Modified"));
	}
  }

  /** Method ModifyEvalTop()
   *  This method will insert/update the top to the Type Log Tops Evaluated DB Table
   *  @param sEval = ACCEPT or REJECT depending on the Button
   */

  private void ModifyEvalTop( String sEval )
  {
	int    iError = 0;
	String sError = "";
	String sUser  = stStruct.stCMN.stUser.sKEY;
	String sName  = txtTops.getText();

	if (sWell.length() < 2)
	{
	  iError = 1;
	  sError = "There is NO Well Information!";
	}
	else if (sUser.length() < 2)
	{
	  iError = 1;
	  sError = "Problem User ID is NOT Present!";
	}
	else if (sZeller.length() < 2)
	{
	  iError = 1;
	  sError = "You must select a valid Zeller Top! \n" +
	           "Select Table or Chart to select a valid Zeller Top";
	}

    if ((dEnd < dStart) && (dEnd > 0.0))
      dEnd = dStart;

    if (iError == 1)
    {
      JOptionPane.showMessageDialog(
          (Component) null, sError, "ERROR", JOptionPane.ERROR_MESSAGE);
    }
    else
    {
	  kgs.io.kgsIOWrite.SaveTopsEvalData( sWell, sUser, sZeller, sName, dStart, dEnd, sEval );

	  if (pNotifier != null)
	     pNotifier.notifyObservers(new String("Evaluator Tops Modified"));

	}
  }

  /* ===================================================================== *
   * ------------------------ DISPLAY DIALOGS ---------------------------- *
   * ===================================================================== */

  /** Method displayChart()
   * <p> This method will display either the Kansas Stratigraphic Chart
   */

  private void displayChart()
  {
    if (pChart != null)
      pChart.close();
    pChart = null;

    pChart = new stratPlotStateFrame( notifier, stStruct.stICS, stStruct.stKGS );
  }

  /** Method displayTable()
   * <p> This method will display either the Kansas Stratigraphic Chart
   */

  private void displayTable()
  {
    if (pTable != null)
      pTable.close();
    pTable = null;

    pTable = new stratListFrame( notifier, stStruct.stKGS );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnTable) { displayTable(); } // Search By Table
    if (event.getSource() == btnChart) { displayChart(); } // Search By Chart

    if (event.getSource() == btnRefresh)
    {
      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Set Working Depth"));
	}

    if (event.getSource() == btnDatumBy)  // Datum By Top Data
    {
System.out.println(sTops);
      if (pNotifier != null)
         pNotifier.notifyObservers(new String("Datum By"));
	}

    if (event.getSource() == btnCopy)  // Copy Top Data
    {
      if (pNotifier != null)
        pNotifier.notifyObservers(new String("Set Copy Depth"));
	}

    if (event.getSource() == btnSave)   { ModifySourceTop(); } // Save Tops Data
    if (event.getSource() == btnDelete) { DeleteSourceTop(); } // Delete Tops Data

    if (event.getSource() == btnAccept) { ModifyEvalTop( "ACCEPT" ); } // Accept Tops Data
    if (event.getSource() == btnReject) { ModifyEvalTop( "REJECT" ); } // Reject Tops Data

    if (btnIcons != null)
    {
      if (stEval != null)
      {
	    for (int i=0; i<stEval.iCount; i++)
	    {
		  if (event.getSource() == btnIcons[i])
		  {
		    dEval = stEval.stItem[i].depthStart;

            if (pNotifier != null)
              pNotifier.notifyObservers(new String("Set Evaluator Depth"));
	  	  }
	    }
  	  }
    }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
    String sMessage = "";
    String sName    = "";
	stratListStruct st  = null;
	stratListStruct stT = null;

    if ((e.getSource() == txtTops) && (iWell > 0) && (stStruct.stCMN.stUser.sKEY.equals(src_kid)))
    {
      sName = txtTops.getText();

      if (sName.length() > 0)
      {
	    st = kgs.io.kgsIORead.ReadByPartialName( sName,
                                                 horizon.strat.io.ReadStratUnitsXMLFile.URL);
        if (st != null)
        {
		  if (st.iCount > 0)
		  {
            if (pTable != null)
              pTable.close();
            pTable = null;

            pTable = new stratListFrame( notifier, st );
		  }
		  else
		  {
  		    sMessage = "No Horizon exists with the Partial Phrase " + sName;
            JOptionPane.showMessageDialog(
              (Component) null, sMessage, "ERROR", JOptionPane.ERROR_MESSAGE);
		  }
		}
		else
		{
		  sMessage = "No Horizon exists with the Partial Phrase " + sName;
          JOptionPane.showMessageDialog(
              (Component) null, sMessage, "ERROR", JOptionPane.ERROR_MESSAGE);
		}
	  }
    }
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("State Stratigraphic Unit Selected")) { getSelection(); }
    if (sArg.equals("Table - State Stratigraphic Unit Selected")) { getTableSelection(); }
  }
}

/** CLASS xsectionTypeLogTopsFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class xsectionTypeLogTopsFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  xsectionTypeLogTopsFrame adaptee;

  xsectionTypeLogTopsFrameFocusAdapter(xsectionTypeLogTopsFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS xsectionTypeLogTopsFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class xsectionTypeLogTopsFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 11/08/2012
 *  @author  John Victorine
 */
